# Russian lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "LLM на устройстве",

    "module_ondevice_llm_config_label": "Модель LLM на устройстве",
    "module_ondevice_llm_config_path_label": "Расположение модели ONNX",
    "module_ondevice_llm_config_path_input_placeholder_text": "Путь к директории модели",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Поле ввода с селектором для указания пути к директории модели, где находятся файлы ONNX.\n"
        "Поддерживаемые модели находятся в формате ONNX, что означает Open Neural Network Exchange, открытый стандарт\n"
        "формата для моделей машинного обучения.",

    "module_ondevice_llm_config_response_temperature_label": "Температура: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Регулирует случайность ответов модели. Высокие значения создают более разнообразные результаты,\n"
        "в то время как низкие значения делают ответы более предсказуемыми.",

    "module_ondevice_llm_config_response_max_tokens_label": "Максимальное количество токенов на ответ",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Устанавливает максимальное количество токенов, которые можно получить в ответе, включая слова и пунктуацию,\n"
        "контролируя длину вывода.",

    "module_ondevice_llm_config_prompt_history_size_label": "Размер истории команд",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Контролирует количество записей в истории команд, которые система сохраняет для справки.\n"
        "Значение ноль позволяет неограниченное количество записей."
}



# French lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "API OpenAI",

    "module_openai_api_label": "API OpenAI",
    "module_openai_api_url_label": "URL de l'API",
    "module_openai_api_url_input_placeholder_text": "URL de l'API",
    "module_openai_api_url_input_accessible_description":
        "L'URL de l'API OpenAI est l'adresse du point de terminaison de l'API, qui peut varier en fonction du service\n"
        "et de la version. L'Assistant AI utilise celle dédiée à la fonctionnalité de chat conversationnel ou\n"
        "aux complétions de texte. Consultez la documentation officielle de l'API OpenAI pour obtenir l'URL actuelle.",
    "module_openai_api_key_label": "Clé API",
    "module_openai_api_key_input_placeholder_text": "Clé API",
    "module_openai_api_key_input_accessible_description":
        "La clé API OpenAI est un jeton secret utilisé pour authentifier les requêtes au point de terminaison de l'API.",
    "module_openai_api_supported_models_label": "Modèles pris en charge",
    "module_openai_api_model_names_combo_placeholder_text": "Choisir un modèle",
    "module_openai_api_model_names_combo_accessible_description":
        "Sélectionnez parmi les modèles pris en charge pour les conversations par chat.",

    "module_openai_api_base_system_prompt_label": "Prompt Système",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "Prompt système de base qui précède chaque demande",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "Un prompt système de base qui précède chaque demande.\n"
        "Généralement, il s'agit de texte simple avec des instructions ou des caractéristiques de rôle.",

    "module_openai_api_base_response_temperature_label": "Température : {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Ajuste l'aléatoire des réponses du modèle. Des valeurs plus élevées produisent des résultats plus variés, "
        "tandis que des valeurs plus faibles rendent les réponses plus prévisibles.",

    "module_openai_api_base_response_max_tokens_label": "Nombre maximum de tokens de réponse",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Le nombre maximum de tokens à recevoir en réponse, tels que des mots et de la ponctuation, "
        "contrôlant la longueur de la sortie.",

    "module_openai_api_config_prompt_history_size_label": "Taille de l'historique des prompts",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Contrôle le nombre d'entrées dans l'historique des prompts que le système conserve pour référence.\n"
        "Une valeur nulle permet un nombre illimité d'entrées."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Create a new directory UI dialog.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDialog, QLabel, QLineEdit, QPushButton, QHBoxLayout, QVBoxLayout, QStyle, QMessageBox
from PySide6.QtGui import QFontMetrics

from . import Settings
from . import Lexemes
from . import ThemeHelper

import os
import logging

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Union  # noqa: F401


class CreateNewDirDialog(QDialog):

    DIR_NAME_MAX_LENGTH = 255

    def __init__(self, base_dir: str, parent=None):
        super().__init__(parent)

        self.parent = parent

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the main window to the dialog
            self.setFont(self.parent.font())

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('create_new_dir_dialog')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='common')

        self.theme_helper = ThemeHelper()

        # Base directory where a new one should be created
        self.base_dir = base_dir

        self.new_dir_name = None  # type: Union[QLineEdit, None]
        self.ok_button = None  # type: Union[QPushButton, None]

        self.init_ui()

    def init_ui(self):

        title = self.lexemes.get('dialog_create_new_dir_title')
        self.setWindowTitle(title)

        new_dir_name_layout = QVBoxLayout()
        new_dir_name_layout.addWidget(QLabel(self.lexemes.get('dialog_create_new_dir_label')))
        self.new_dir_name = QLineEdit()
        self.new_dir_name.setMaxLength(self.DIR_NAME_MAX_LENGTH)  # set maximum length of the input text
        self.new_dir_name.setPlaceholderText(self.lexemes.get('dialog_create_new_dir_input_placeholder_text'))
        self.new_dir_name.setEchoMode(QLineEdit.EchoMode.Normal)
        self.new_dir_name.setFocus()
        new_dir_name_layout.addWidget(self.new_dir_name)

        buttons_layout = QHBoxLayout()

        self.ok_button = QPushButton(self.lexemes.get('dialog_create_new_dir_button_ok'))
        self.ok_button.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_DialogOkButton))
        self.ok_button.clicked.connect(self.validate_and_accept)  # or `self.accept`
        self.ok_button.setEnabled(False)

        # https://doc.qt.io/qt-6/qt.html#FocusPolicy-enum
        self.cancel_button = QPushButton(self.lexemes.get('dialog_create_new_dir_button_cancel'),
                                         focusPolicy=Qt.FocusPolicy.NoFocus)
        self.cancel_button.setIcon(self.style().standardIcon(QStyle.StandardPixmap.SP_DialogCancelButton))
        self.cancel_button.clicked.connect(self.reject)

        buttons_layout.addWidget(self.cancel_button)
        buttons_layout.addWidget(self.ok_button)

        layout = QVBoxLayout()
        layout.addLayout(new_dir_name_layout)
        layout.addLayout(buttons_layout)

        self.setLayout(layout)

        self.adjust_minimum_width(title)

        self.new_dir_name.textChanged.connect(self.check_input_length)

    def check_input_length(self):
        dir_name_length = len(self.new_dir_name.text())

        # Enable OK button only if field have non-zero length
        self.ok_button.setEnabled(dir_name_length and 0 < dir_name_length <= self.DIR_NAME_MAX_LENGTH
                                  or dir_name_length == 0)

    def validate_and_accept(self):
        dir_name = self.new_dir_name.text()

        if not dir_name:
            QMessageBox.warning(self,
                                self.lexemes.get('dialog_create_new_dir_warning_empty_title'),
                                self.lexemes.get('dialog_create_new_dir_warning_empty_text'))
            return

        if len(dir_name) > self.DIR_NAME_MAX_LENGTH:
            # This statement shouldn't be reachable anyway
            QMessageBox.warning(self,
                                self.lexemes.get('dialog_create_new_dir_warning_too_long_title'),
                                self.lexemes.get('dialog_create_new_dir_warning_too_long_text',
                                                 symbols=self.DIR_NAME_MAX_LENGTH))
            return

        if self.create_new_dir_dialog_callback(new_dir_name=dir_name):
            # Accept dialog result
            self.accept()
        else:
            return

    def create_new_dir_dialog_callback(self, new_dir_name: str) -> bool:
        """
        Actions to perform after create a new directory dialogue.
        """

        # New dir path
        new_dir_path = os.path.join(self.base_dir, new_dir_name)

        self.logger.debug('Creating a new dir "%s" within the "%s" dialog callback'
                          % (new_dir_path, self.base_dir))

        try:
            os.mkdir(new_dir_path)
            self.logger.debug(f"New directory '{new_dir_path}' created successfully.")
            return True
        except FileExistsError:  # if not os.path.exists(new_dir_path):
            self.logger.debug(f"Directory '{new_dir_path}' already exists.")
            # Show error message
            self.parent.message_box(self.lexemes.get('dialog_create_new_dir_error_existed'), icon_type=2)  # noqa
            return False
        except OSError as e:
            self.logger.debug(f"Failed to create directory '{new_dir_path}': {e}")
            # Show error message
            self.parent.message_box(  # noqa
                self.lexemes.get('dialog_create_new_dir_error', base_dir=self.base_dir), icon_type=2)
            return False

    def adjust_minimum_width(self, title):
        # Calculate width required for the title text
        font_metrics = QFontMetrics(self.font())
        title_width = font_metrics.horizontalAdvance(title)

        # Set minimum width for the dialog
        self.setMinimumWidth(title_width + 150)  # Add some padding

    def set_font_size(self, widget, percent):
        default_font_size = self.font().pointSizeF()
        font = widget.font()
        font.setPointSizeF(default_font_size * (percent / 100))
        widget.setFont(font)



# Dutch lexemes settings_dialog.py
lexemes = {
    # Settings dialog
    "window_title": "Instellingen",

    "button_close": "Sluiten",

    "tab_general": "Algemeen",
    "tab_editor_config": "Editor",
    "tab_viewer_config": "Viewer",
    "tab_ai_config": "AI Configuratie",

    "general_app_config_label": "App configuratie",
    "general_app_language_label": "Taal",
    "general_app_language_combo_placeholder_text": "Kies een taal",
    "general_app_language_combo_accessible_description": "Interface taal van de app",
    "general_app_theme_label": "Thema",
    "general_app_theme_combo_placeholder_text": "Kies een thema",
    "general_app_theme_combo_accessible_description": "Interface thema van de app",
    "general_app_main_menu_label": "Hoofdmenu",
    "general_app_main_menu_checkbox": "Toon hoofdmenu",
    "general_app_main_menu_checkbox_accessible_description": "Toon het hoofddropdownmenu van de app",
    "general_app_font_size_label": "Lettergrootte: {size}pt",
    "general_app_font_size_slider_accessible_description": "Pas de globale lettergrootte van de app aan",

    "general_statusbar_label": "Statusbalk",
    "general_statusbar_show_global_cursor_position_checkbox": "Toon globale cursorpositie",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Toon de globale cursorpositie in de statusbalk",

    "editor_config_label": "Editor Configuratie",
    "editor_config_show_line_numbers_checkbox": "Toon regelnummers",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Toon regelnummers in de editor",

    "viewer_config_label": "Viewer Configuratie",
    "viewer_config_process_emojis_checkbox": "Zet tekstemoji's om naar grafische afbeeldingen",
    "viewer_config_process_emojis_checkbox_accessible_description": "Zet tekstemoji's om naar grafische afbeeldingen",
    "viewer_config_highlight_todos_checkbox": "Markeer TODO's",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Benadruk TODO-tags binnen de tekst",
    "viewer_config_open_link_confirmation_checkbox": "Bevestiging vereist om links te openen",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Vraag om bevestiging voordat links worden geopend",
    "viewer_config_save_resources_checkbox": "Auto-save externe afbeeldingen naar schijf",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Sla automatisch kopieën van externe afbeeldingen op schijf op voor offline toegang.",

    "ai_config_inference_module_label": "Inferentiemodule",
    "ai_config_inference_module_names_combo_label": "Actieve Inferentiemodule",
    "ai_config_inference_module_names_combo_placeholder_text": "Kies een module",
    "ai_config_inference_module_names_combo_accessible_description":
        "Selecteer uit beschikbare AI-inferentiemodules om te werken met de AI-assistent.\n"
        "Opties omvatten grote taalmodellen (LLM) met realtime verwerking of API-gebaseerde functionaliteiten.",

    "ai_config_base_label": "Basisparameters",
    "ai_config_multi_turn_dialogue_checkbox": "Meerdere gespreksbeurten met geheugen",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "Schakel een dialoog in met meerdere gespreksbeurten die de vorige prompt bewaart voor gespreksgeheugen.\n"
        "Als het is uitgeschakeld, beïnvloeden alleen het nieuwe bericht en de systeemprompt de reactie.",
    "ai_config_convert_to_md_checkbox": "Converteer het resultaat naar Markdown",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "Converteer het uitvoerbericht naar Markdown-indeling.",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Example of an etree extension that allows processing and modifying the document's tree.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from xml.etree import ElementTree

from markdown import Markdown
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

import re
import logging


class ElementTreeExtension(Extension):
    """
    Custom markdown extension.
    """

    logger = logging.getLogger('tree_extension')

    def extendMarkdown(self, md: Markdown) -> None:
        self.logger.debug('%s extension engaged' % self.__class__.__qualname__)
        # Registering extension
        md.registerExtension(self)
        # noinspection SpellCheckingInspection
        md.treeprocessors.register(ElementTreeProcessor(md), 'notolog_extension', 1)


class ElementTreeProcessor(Treeprocessor):
    """
    Custom markdown extension tree processor.
    """

    logger = logging.getLogger('tree_processor')

    def run(self, root):
        # Customize elements tree here
        for idx, elem in enumerate(root.iter()):
            self.logger.debug('Process elements > %d: %s' % (idx, elem.tag))
            # Process <table> elements
            if elem.tag == 'table':
                self.process_table(elem)
            if elem.tag in ['th', 'td']:
                self.process_table_td(elem)
            # Process <img> elements
            if elem.tag == 'img':
                self.process_image(elem)
            # Process <p> elements
            if elem.tag == 'p':
                self.process_paragraph(elem)
            # noinspection SpellCheckingInspection
            """
            Works when `codehilite` is switched off:
            if elem.tag == 'pre':
                for id_xy, c_elem in enumerate(elem):
                    if c_elem.tag == 'code':
                        c_elem.set('style', 'background-color: #373737; color: #f5f5f5; padding: 10px;')
            """

    def get_index(self, root, element):
        for idx, child in enumerate(root):
            if element == child:
                return idx
        else:
            raise ValueError("No inner '%s' tag found in '%s'" % (element.tag, root.tag))

    def process_table(self, elem):
        if not elem.get('class'):
            self.logger.debug(f'Table without class found {elem}')
            elem.set('class', '_nl_tbl')
            elem.set('cellpadding', '0')
            elem.set('cellspacing', '0')

    def process_table_td(self, elem):
        # Replace 'style' with 'class' as css doesn't work otherwise
        if elem.get('style') and elem.get('style').startswith('text-align'):
            # re = QRegularExpression(r'^text-align:\s?(.*?);$')
            # match = re.match(elem.get('style'))
            # if match.capturedTexts() and match.captured(1):
            #    ...
            pattern = re.compile(r'^text-align:\s?(.*?);$')
            match = pattern.search(elem.get('style'))
            if match:
                del elem.attrib['style']
                elem.set('class', match.group(1))

    def process_image(self, elem):
        self.logger.debug(f'Image element {elem}')
        # elem.set('title', '...')
        # elem.set('onerror', "this.onerror=null;this.src='...';")

    def process_paragraph(self, elem):
        # Check inner elements
        if not (len(elem) > 0):
            return
        # Process inner elements
        for id_xy, c_elem in enumerate(elem):
            self.logger.debug('Inner element > %d: %s' % (id_xy, c_elem.tag))
            # Example of how to update elements tree
            if c_elem.tag == 'a' and False:
                # Skip element that was already re-inserted
                if c_elem.get('class') == '_re-inserted':
                    continue
                # Add break space before the element
                br_str = "<br/>"
                br_elem = ElementTree.fromstring(br_str)
                # Remove `a` element
                elem.remove(c_elem)
                # Insert `br` element
                elem.insert(id_xy, br_elem)
                # Setup class as a flag to determine re-inserted `a` element
                c_elem.set('class', '_re-inserted')
                # Re-insert the `a` element with a new class
                elem.insert(id_xy + 1, c_elem)



# Chinese lexemes ai_assistant.py
lexemes = {
    # AI 助手
    "dialog_title": "向 AI 提问",
    "dialog_prompt_input_placeholder_text": "请求",
    "dialog_prompt_input_accessible_description": "输入请求以提供给 AI 助手上下文",
    "dialog_response_output_placeholder_text": "响应数据",
    "dialog_response_output_accessible_description": "响应数据将出现在此字段",
    "dialog_response_output_notice_empty_text": "提供文本输入以获得大约...",

    "dialog_usage_module_label": "模块",
    "dialog_usage_model_label": "模型",
    "dialog_usage_tokens_label": "令牌",
    "dialog_usage_tokens_prompt": "提示：{tokens}",
    "dialog_usage_tokens_answer": "答案：{tokens}",
    "dialog_usage_tokens_total": "总计：{tokens}",

    "dialog_button_send_request": "发送请求",
    "dialog_button_stop_request": "停止",
    "dialog_button_save_history": "保存对话历史",

    "dialog_message_copy_tooltip": "复制文本",
    "dialog_message_copied_tooltip": "已复制",

    "dialog_error_loading_model": "加载自定义模型错误",
}



# Turkish lexemes ai_assistant.py
lexemes = {
    "dialog_title": "AI'ya İstek",
    "dialog_prompt_input_placeholder_text": "İstek",
    "dialog_prompt_input_accessible_description": "AI yardımcısına bağlam sağlamak için istek girin",
    "dialog_response_output_placeholder_text": "Yanıt verisi",
    "dialog_response_output_accessible_description": "Yanıt verisi bu alanda görünecek",
    "dialog_response_output_notice_empty_text": "Yaklaşık bir yanıt almak için metin girişi sağlayın",

    "dialog_usage_module_label": "MODÜL",
    "dialog_usage_model_label": "MODEL",
    "dialog_usage_tokens_label": "TOKENLAR",
    "dialog_usage_tokens_prompt": "istek: {tokens}",
    "dialog_usage_tokens_answer": "cevap: {tokens}",
    "dialog_usage_tokens_total": "toplam: {tokens}",

    "dialog_button_send_request": "İstek Gönder",
    "dialog_button_stop_request": "Durdur",
    "dialog_button_save_history": "Sohbet geçmişini kaydet",

    "dialog_message_copy_tooltip": "Metni kopyala",
    "dialog_message_copied_tooltip": "Kopyalandı",

    "dialog_error_loading_model": "Özel model yüklenirken hata",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.helpers.theme_helper import ThemeHelper
from notolog.theme import Theme
from notolog.settings import Settings
from notolog.app_config import AppConfig

from logging import Logger

from . import test_core_app  # noqa: F401

import os
import pytest
import logging


class TestThemeHelper:

    @pytest.fixture(autouse=True)
    def test_settings_fixture(self, request, test_core_app):  # noqa: F811 redefinition of unused 'test_app'
        # The test_app fixture sets up the app environment explicitly.
        setting = Settings()
        # Retrieve parameter values from the test request.
        setting.app_theme = request.param
        yield setting

    @pytest.fixture(autouse=True)
    def test_obj_theme_helper(self, mocker, test_settings_fixture):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Mock AppConfig's get_logger_level method to suppress logging during tests.
        mocker.patch.object(AppConfig, 'get_logger_level', return_value=logging.NOTSET)

        """
        If themes dir is set read themes from there.
        For test purposes to check theme files are readable.
        """
        test_themes_dir = os.path.join(os.path.dirname(__file__), 'test_themes')
        mocker.patch.object(Theme, 'get_themes_dir', return_value=test_themes_dir)

        # Allow to change theme name during test cycle with no caching
        AppConfig().set_test_mode(True)

        yield ThemeHelper()

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        """
        Fixture to pass params and get expected results.
        This method is used only for passing params via pytest fixture.
        """

        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.mark.parametrize(
        "test_settings_fixture, test_obj_theme_helper, test_exp_params_fixture",
        [
            ((None), (None), ('default')),
            ((''), (None), ('default')),
            (('default'), (None), ('default')),  # Because of default() method
            (('DEFAULT'), (None), ('default')),
            (('anyval'), (None), ('default')),
        ],
        indirect=True
    )
    def test_theme_helper_init(self, test_settings_fixture, test_obj_theme_helper: ThemeHelper,
                               test_exp_params_fixture):
        """
        Test and check initial params are exist in the testing object.
        """
        assert isinstance(test_obj_theme_helper.logger, Logger)

        exp_theme = test_exp_params_fixture

        assert hasattr(test_obj_theme_helper, 'theme')
        assert isinstance(test_obj_theme_helper.theme, Theme)
        assert isinstance(test_obj_theme_helper.settings, Settings)
        assert test_obj_theme_helper.theme.theme == exp_theme

    @pytest.mark.parametrize(
        "test_settings_fixture, test_obj_theme_helper, test_exp_params_fixture",
        [
            (None, None, (None, None, None)),
            ('default', None, ('test_red', None, 0xFF0000)),
            ('default', None, ('test_green', True, '#00FF00')),
            ('default', None, ('test_blue', False, 0x0000FF)),
            ('default', None, ('test_undefined', None, None)),
            # 'noir_dark' is real as it can be checked with Themes enum
            ('noir_dark', None, ('test_non_red', False, 0x00FFFF)),
            ('noir_dark', None, ('test_light', True, '#FFFFFF')),
            ('test_undefined', None, (None, None, None)),
            ('test_undefined', None, ('undefined_name', True, None)),
        ],
        indirect=True
    )
    def test_themes_get_color(self, test_settings_fixture, test_obj_theme_helper, test_exp_params_fixture):
        """
        Test getter for correctly loaded and really existent theme's colors.
        """
        param_name, param_css_format, exp_result = test_exp_params_fixture

        assert test_obj_theme_helper.get_color(param_name, param_css_format) == exp_result

    @pytest.mark.parametrize(
        "test_settings_fixture, test_obj_theme_helper, test_exp_params_fixture",
        [
            (None, None, (None, None)),
            ('default', None, ('test_smth', None)),
            ('default', None, ('test_styles', 'QTextEdit {}')),
            ('noir_dark', None, ('test_smth', 'QTextEdit {}')),
        ],
        indirect=True
    )
    def test_themes_get_css(self, test_settings_fixture, test_obj_theme_helper, test_exp_params_fixture):
        """
        Test getter for correctly loaded and really existent theme's css files.
        """
        param_css_name, exp_result = test_exp_params_fixture

        assert test_obj_theme_helper.get_css(param_css_name) == exp_result



# Spanish lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "LLM en Dispositivo",

    "module_ondevice_llm_config_label": "Modelo LLM en Dispositivo",
    "module_ondevice_llm_config_path_label": "Ubicación del Modelo ONNX",
    "module_ondevice_llm_config_path_input_placeholder_text": "Ruta al directorio del modelo",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Un campo de entrada con un selector para especificar la ruta al directorio del modelo donde se encuentran\n"
        "los archivos ONNX. Los modelos compatibles están en formato ONNX, que representa el intercambio de redes\n"
        "neuronales abiertas, un estándar abierto para los formatos de modelos de aprendizaje automático.",

    "module_ondevice_llm_config_response_temperature_label": "Temperatura: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Ajusta la aleatoriedad de las respuestas del modelo. Valores más altos producen salidas más variadas,\n"
        "mientras que valores más bajos hacen que las respuestas sean más predecibles.",

    "module_ondevice_llm_config_response_max_tokens_label": "Máximo de Tokens de Respuesta",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Establece el número máximo de tokens que se recibirán en una respuesta, como palabras y puntuación,\n"
        "controlando la longitud de la salida.",

    "module_ondevice_llm_config_prompt_history_size_label": "Tamaño del Historial de Prompts",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Controla el número de entradas en el historial de prompts que el sistema retiene para referencia.\n"
        "Un valor de cero permite entradas ilimitadas."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends QLabel functionality by adding an icon with a customizable hint tooltip.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, QSize
from PySide6.QtWidgets import QLabel, QToolButton, QHBoxLayout
from PySide6.QtGui import QIcon, QColor, QPixmap

import logging

from ..app_config import AppConfig
from ..settings import Settings
from ..helpers.theme_helper import ThemeHelper

from typing import TYPE_CHECKING, Union

if TYPE_CHECKING:
    from ..notolog_editor import NotologEditor  # noqa: F401


class LabelWithHint(QLabel):
    def __init__(self, parent, text=None, action=None, theme_icon=None, tooltip: Union[dict, tuple] = None):
        super().__init__(parent)

        self.parent = parent  # type: NotologEditor

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the main window to the dialog
            self.setFont(self.parent.font())

        self.logger = logging.getLogger('label_with_hint')

        self.text = text
        self.action = action
        self.tooltip = tooltip
        self.theme_icon = theme_icon

        # Default icon
        if not self.theme_icon:
            self.theme_icon = 'question-circle.svg'

        self.theme_helper = ThemeHelper()

        self.settings = Settings()

        self.layout = None  # type: Union[QHBoxLayout, None]
        self.text_label = None  # type: Union[QLabel, None]
        self.icon_button = None  # type: Union[QToolButton, None]

        self.init_ui()

    def init_ui(self):
        # Main layout
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(5)
        self.setLayout(self.layout)

        # Create icon button
        self.icon_button = QToolButton()
        self.icon_button.setCursor(Qt.CursorShape.WhatsThisCursor)
        self.icon_button.setStyleSheet("border: none;")  # Remove button border
        # Load icon
        self.load_icon()
        # Set the tooltip text for the icon if applicable
        if self.tooltip:
            self.load_tooltip(self.tooltip)

        # Set the click action for the icon
        if self.action:
            self.icon_button.clicked.connect(self.action)

        # Create the text label
        self.text_label = QLabel(self.text)
        self.text_label.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)

        # Add icon and text to the layout
        self.layout.addWidget(self.text_label)
        self.layout.addWidget(self.icon_button, alignment=Qt.AlignmentFlag.AlignRight)

    def load_icon(self):
        """
        Load and set the icon for the button.
        """

        # Retrieve the icon color from the theme
        icon_color = self.theme_helper.get_color('settings_dialog_hint_icon_color')

        # Get the icon path with the specified theme and color
        icon_path = self.theme_helper.get_icon(theme_icon=self.theme_icon, color=QColor(icon_color))

        # Determine the text height using font metrics
        font_metrics = self.fontMetrics()
        text_height = font_metrics.height()

        # Create a QPixmap for the icon, scaled to match the text height
        pixmap = QPixmap(icon_path.pixmap(QSize(text_height, text_height)))
        pixmap.scaled(text_height, text_height,
                      Qt.AspectRatioMode.KeepAspectRatio, Qt.TransformationMode.SmoothTransformation)

        # Set the icon on the button
        self.icon_button.setIcon(QIcon(pixmap))

    def load_tooltip(self, tooltip):
        """
        Extract and set the tooltip data for the button.

        Args:
            tooltip (dict or tuple): Tooltip data containing the lexeme and text.
        """

        # Extract tooltip data based on type
        if isinstance(tooltip, dict):  # Handle dict
            tooltip_lexeme, tooltip_text = next(iter(tooltip.items()))
        elif isinstance(tooltip, tuple):  # Handle tuple
            tooltip_lexeme, tooltip_text = tooltip
        else:
            raise TypeError(f"Expected a dict or tuple: {tooltip}")

        # Store the tooltip lexeme for potential future use
        self.setProperty("tooltip_lexeme", tooltip_lexeme)

        # Set the tooltip text and appearance
        self.set_tooltip(tooltip_text)

    def set_tooltip(self, tooltip_text):
        """
        Set the tooltip text and style.

        Args:
            tooltip_text (str): The text to display in the tooltip.
        """

        # Set the tooltip text
        self.icon_button.setToolTip(tooltip_text)

        # Customize the tooltip font size
        font_size = AppConfig().get_font_size()
        self.setStyleSheet("QToolTip { font-size: %dpt; }" % font_size)



# Japanese lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "デバイス上のLLM",

    "module_ondevice_llm_config_label": "デバイス上のLLMモデル",
    "module_ondevice_llm_config_path_label": "ONNXモデルの位置",
    "module_ondevice_llm_config_path_input_placeholder_text": "モデルディレクトリへのパス",
    "module_ondevice_llm_config_path_input_accessible_description":
        "ONNXファイルが位置するモデルディレクトリへのパスを指定するセレクター付き入力フィールド。\n"
        "サポートされるモデルはONNX形式で、これはオープンニューラルネットワークエクスチェンジの略で、\n"
        "機械学習モデルのためのオープンスタンダードフォーマットです。",

    "module_ondevice_llm_config_response_temperature_label": "温度: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "モデルの応答のランダム性を調整します。高い値はより多様な出力を生み出し、\n"
        "低い値はより予測可能な応答をもたらします。",

    "module_ondevice_llm_config_response_max_tokens_label": "最大応答トークン数",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "応答で受け取るトークンの最大数を設定します。これには単語や句読点が含まれ、\n"
        "出力の長さを制御します。",

    "module_ondevice_llm_config_prompt_history_size_label": "プロンプト履歴のサイズ",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "システムが参照用に保持するプロンプト履歴のエントリ数を制御します。\n"
        "ゼロの値は無制限のエントリを許可します。"
}



# Japanese lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "モジュール llama.cpp",

    "module_llama_cpp_config_label": "モジュール llama.cpp",
    "module_llama_cpp_config_path_label": "モデルの位置",
    "module_llama_cpp_config_path_input_placeholder_text": "モデルパスを選択または入力してください",
    "module_llama_cpp_config_path_input_accessible_description":
        "ローカルモデルのパスを指定するためのセレクター付き入力フィールドです。GGUF形式のモデルをサポートし、\n"
        "GGMLおよびGGMLベースのエクゼキューターで使用されるモデルを格納するために最適化されたバイナリファイル形式です。",
    "module_llama_cpp_config_path_input_filter_text": "GGUFファイル",

    "module_llama_cpp_config_context_window_label": "コンテキストウィンドウのサイズ",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "モデルがレスポンスを生成するために考慮するトークンの数を設定します。どれだけの前のコンテキストが使用されるかを制御します。",

    "module_llama_cpp_chat_formats_label": "チャット形式",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "チャット形式を選択してください",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "モデル会話に使用される形式を選択するためのドロップダウンメニューです。",

    "module_llama_cpp_config_system_prompt_label": "システムプロンプト",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "システムプロンプトのテキストを入力してください",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "モデルの応答を導くシステムプロンプトを入力するためのテキストフィールドです。",

    "module_llama_cpp_config_response_temperature_label": "応答の温度: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "モデルの応答のランダム性を調整します。高い値はより多様な出力を生成し、\n"
        "低い値はより予測可能な応答をもたらします。",

    "module_llama_cpp_config_response_max_tokens_label": "応答あたりの最大トークン数",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "実際のコンテキストウィンドウの制限までモデルの応答中のトークン数を制限します。\n"
        "ゼロ値はコンテキストウィンドウの容量を想定します。",

    "module_llama_cpp_config_prompt_history_size_label": "プロンプト履歴のサイズ",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "システムによって参照のために保持されるプロンプト履歴のエントリの数を制御します。\n"
        "ゼロ値は無制限のエントリを可能にします。"
}



# Dutch lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Module llama.cpp",

    "module_llama_cpp_model_exception": "Model niet gevonden fout: {error_msg}",
    "module_llama_cpp_task_exception": "Kan modelgenerator niet initialiseren: {error_msg}",
}



# Turkish lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Modül llama.cpp",

    "module_llama_cpp_config_label": "Modül llama.cpp",
    "module_llama_cpp_config_path_label": "Model Yerleşimi",
    "module_llama_cpp_config_path_input_placeholder_text": "Model yolu seçin veya girin",
    "module_llama_cpp_config_path_input_accessible_description":
        "Yerel modelin yolunu belirtmek için bir seçiciye sahip giriş alanı. GGUF formatındaki modelleri destekler,\n"
        "GGML ile kullanılan ve GGML tabanlı yürütücüler için optimize edilmiş bir ikili dosya formatıdır.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF Dosyaları",

    "module_llama_cpp_config_context_window_label": "Bağlam Penceresi Boyutu",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Modelin yanıtları üretmek için dikkate aldığı token sayısını belirler.\n"
        "Ne kadar önceki bağlamın kullanıldığını kontrol eder.",

    "module_llama_cpp_chat_formats_label": "Sohbet Formatları",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Bir sohbet formatı seçin",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Model konuşmaları için kullanılan formatı seçmek için açılır menü.",

    "module_llama_cpp_config_system_prompt_label": "Sistem İstem",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Sistem istem metnini girin",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Model yanıtlarını yönlendiren sistem istemlerini girmek için metin alanı.",

    "module_llama_cpp_config_response_temperature_label": "Yanıt Sıcaklığı: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Modelin yanıtlarının rastgeleliğini ayarlar. Daha yüksek değerler daha çeşitli çıktılar üretir,\n"
        "daha düşük değerler ise daha öngörülebilir yanıtlar verir.",

    "module_llama_cpp_config_response_max_tokens_label": "Yanıt Başına Maksimum Token",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Modelin yanıtlarındaki token sayısını gerçek bağlam penceresi limitine kadar sınırlar.\n"
        "Sıfır değer, bağlam penceresinin kapasitesini varsayar.",

    "module_llama_cpp_config_prompt_history_size_label": "İstem Geçmişi Boyutu",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Sistem tarafından referans için tutulan istem geçmişindeki giriş sayısını kontrol eder.\n"
        "Sıfır değer sınırsız girişe izin verir."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: File changes observer.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QFileSystemWatcher, QDir

import os
import logging


class FileSystemWatcher:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        if getattr(self, '_initialized', False):
            return
        self._initialized = True

        self.logger = logging.getLogger('file_system_watcher')

        self.dir = None

        self.files = set()

        self.watcher = QFileSystemWatcher()
        self.watcher.directoryChanged.connect(self.on_dir_changed)
        self.watcher.fileChanged.connect(self.on_file_changed)

    def update_files(self):
        if self.dir:
            files = set(self.dir.entryList())
            files.discard(".")  # Ignore current directory entry
            files.discard("..")  # Ignore parent directory entry
            self.files.update(files)

    def watch(self, directory):
        # Disconnect signals from the previous directory
        if self.dir:
            self.watcher.directoryChanged.disconnect()
            self.watcher.fileChanged.disconnect()

        # Clear the previous directory's file list
        self.files.clear()

        # Set up the QFileSystemWatcher to monitor the new directory
        self.dir = QDir(directory)
        self.watcher.addPath(directory)

        # Connect signals for the new directory
        self.watcher.directoryChanged.connect(self.on_dir_changed)
        self.watcher.fileChanged.connect(self.on_file_changed)

        self.update_files()

    def on_dir_changed(self, path):
        new_files = set(self.dir.entryList())

        new_files.discard(".")  # Ignore current dir
        new_files.discard("..")  # Ignore parent dir

        added_files = new_files - self.files
        removed_files = self.files - new_files

        renamed_files = set()
        for added_file in added_files:
            potential_renamed_file = os.path.join(self.dir.path(), added_file)
            if os.path.exists(potential_renamed_file):
                renamed_files.add((added_file, potential_renamed_file))

        if added_files:
            self.logger.debug(f"New file(s) added: {', '.join(added_files)}")
        if removed_files:
            self.logger.debug(f"File(s) removed: {', '.join(removed_files)}")
        if renamed_files:
            self.logger.debug(f"File(s) renamed: {', '.join([f[0] for f in renamed_files])}")

        self.files = new_files

    def on_file_changed(self, path):
        self.logger.debug(f"File changed: {path}")



# Swedish lexemes settings_dialog.py
lexemes = {
    # Settings dialog
    "window_title": "Inställningar",

    "button_close": "Stäng",

    "tab_general": "Allmänt",
    "tab_editor_config": "Redigerare",
    "tab_viewer_config": "Visare",
    "tab_ai_config": "AI-konfiguration",

    "general_app_config_label": "Appkonfiguration",
    "general_app_language_label": "Språk",
    "general_app_language_combo_placeholder_text": "Välj ett språk",
    "general_app_language_combo_accessible_description": "Appens gränssnittsspråk",
    "general_app_theme_label": "Tema",
    "general_app_theme_combo_placeholder_text": "Välj ett tema",
    "general_app_theme_combo_accessible_description": "Appens gränssnittstema",
    "general_app_main_menu_label": "Huvudmeny",
    "general_app_main_menu_checkbox": "Visa huvudmeny",
    "general_app_main_menu_checkbox_accessible_description": "Visa appens huvuddropdownmeny",
    "general_app_font_size_label": "Teckenstorlek: {size}pt",
    "general_app_font_size_slider_accessible_description": "Justera appens globala teckenstorlek",

    "general_statusbar_label": "Statusfält",
    "general_statusbar_show_global_cursor_position_checkbox": "Visa global muspekareposition",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Visa den globala muspekarepositionen i statusfältet",

    "editor_config_label": "Redigerarkonfiguration",
    "editor_config_show_line_numbers_checkbox": "Visa radnummer",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Visa radnummer i redigeraren",

    "viewer_config_label": "Visarkonfiguration",
    "viewer_config_process_emojis_checkbox": "Konvertera textemojis till grafik",
    "viewer_config_process_emojis_checkbox_accessible_description": "Konvertera textemojis till grafiska representationer",
    "viewer_config_highlight_todos_checkbox": "Markera TODOs",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Markera TODO-taggar i texten",
    "viewer_config_open_link_confirmation_checkbox": "Kräv bekräftelse för att öppna länkar",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Begär bekräftelse innan länkar öppnas",
    "viewer_config_save_resources_checkbox": "Spara automatiskt externa bilder på disk",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Sparar automatiskt kopior av externa bilder på disken för offline-åtkomst",

    "ai_config_inference_module_label": "Inferensmodul",
    "ai_config_inference_module_names_combo_label": "Aktiv inferensmodul",
    "ai_config_inference_module_names_combo_placeholder_text": "Välj modul",
    "ai_config_inference_module_names_combo_accessible_description":
        "Välj bland tillgängliga AI-inferensmoduler för att arbeta med AI-assistenten.\n"
        "Alternativen inkluderar stora språkmodeller (LLM) med realtidsbearbetning eller API-baserade funktioner.",

    "ai_config_base_label": "Grundparametrar",
    "ai_config_multi_turn_dialogue_checkbox": "Fleromgångsdialog med samtalsminne",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "Aktivera en fleromgångsdialog som bevarar den senaste inmatningen för samtalsminne.\n"
        "När den är avstängd påverkar endast det nya meddelandet och systemuppmaningen svaret.",
    "ai_config_convert_to_md_checkbox": "Konvertera resultatet till Markdown",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "Konvertera utgångsmeddelandet till Markdown-format.",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the base module.
- Functionality: Manages the loading of modules and performs integrity checks.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import os
import logging
import importlib

from threading import Lock


class Modules:

    _instance = None  # Singleton instance
    _lock = Lock()

    modules: dict = {}

    def __new__(cls, *args, **kwargs):
        # Overriding __new__ to control the instantiation process
        if not cls._instance:
            with cls._lock:
                # Create the instance if it doesn't exist
                cls._instance = super(Modules, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        # Prevent re-initialization if the instance is already set up.
        if hasattr(self, 'logger'):
            return

        # Use a lock to ensure initialization is thread-safe and atomic.
        with self._lock:
            # Double-check to prevent race conditions during initialization.
            if hasattr(self, 'logger'):
                return

            # Initialize the QObject part only once
            super(Modules, self).__init__()

            self.logger = logging.getLogger('modules')

            self.modules = self.load_modules()

    def load_modules(self) -> dict:
        modules_dir = os.path.dirname(os.path.realpath(__file__))
        modules_list = {}
        for module_name in os.listdir(modules_dir):
            if module_name == '__pycache__':
                continue
            module_dir = os.path.join(modules_dir, module_name)
            if os.path.isdir(module_dir) and os.path.isfile(os.path.join(module_dir, 'module_core.py')):
                # Dynamically import a module
                module = self.import_module(module_name)
                # Check the module is available
                if hasattr(module, 'is_available') and module.is_available():
                    modules_list.update({module_name: module})
        return modules_list

    def get_by_extension(self, extension_name) -> list:
        # Get all modules match the extension name
        return [module for module in self.modules.values()
                if any(extension_name == _name for _name in module.ModuleCore.extensions)]

    def get_by_name(self, module_name):
        for module in self.modules.values():
            if module_name in str(module) and module.is_available():
                return module
        return None

    def import_module(self, module_name):
        # Check imported module
        if hasattr(self, 'modules') and module_name in self.modules.keys():
            return self.modules.get(module_name)

        # Try to import module
        try:
            module = importlib.import_module(f'notolog.modules.{module_name}')
            self.logger.info(f"Module '{module_name}' has been imported.")
            self.modules.update({module_name: module})
            return module
        except ImportError as e:
            self.logger.error(f"Failed to import module: {e}")
            return None

    def create(self, module, class_name: str = 'ModuleCore', *args, **kwargs):
        try:
            class_ = getattr(module, class_name)
            instance = class_(*args, **kwargs)
            self.logger.debug(f"Instance of '{module.__name__}.{class_name}' created.")
            return instance
        except AttributeError:
            self.logger.warning(f"Class '{module.__name__}.{class_name}' not found in the module.")
            return None
        except Exception as e:
            self.logger.error(f"Failed to create instance of '{module.__name__}.{class_name}': {e}")
            return None



# French lexemes main_menu.py
lexemes = {
    "group_file_label": "Fichier",
    "group_edit_label": "Éditer",
    "group_tools_label": "Outils",
    "group_help_label": "Aide",

    "actions_file_label_new_document": "Nouveau document",
    "actions_file_accessible_name_new_document": "Nouveau document",
    "actions_file_label_open": "Ouvrir",
    "actions_file_accessible_name_open": "Ouvrir le document",
    "actions_file_label_save": "Enregistrer",
    "actions_file_accessible_name_save": "Enregistrer le document",
    "actions_file_label_save_as": "Enregistrer sous",
    "actions_file_accessible_name_save_as": "Enregistrer une copie du document",
    "actions_file_label_settings": "Paramètres",
    "actions_file_accessible_name_settings": "Paramètres de l'application",
    "actions_file_label_reset_settings": "Réinitialiser les paramètres",
    "actions_file_accessible_name_reset_settings": "Réinitialiser les paramètres",
    "actions_file_label_exit": "Quitter",
    "actions_file_accessible_name_exit": "Quitter l'application",

    "actions_edit_edit_mode": "Modifier",
    "actions_edit_accessible_name_edit_mode": "Modifier le fichier",
    "actions_edit_source_mode": "Source",
    "actions_edit_accessible_name_source_mode": "Code source",
    "actions_edit_bold": "Gras",
    "actions_edit_accessible_name_bold": "Format de texte en gras",
    "actions_edit_italic": "Italique",
    "actions_edit_accessible_name_italic": "Format de texte en italique",
    "actions_edit_underline": "Souligner",
    "actions_edit_accessible_name_underline": "Format de texte souligné",
    "actions_edit_strikethrough": "Barré",
    "actions_edit_accessible_name_strikethrough": "Format de texte barré",
    "actions_edit_blockquote": "Citation",
    "actions_edit_accessible_name_blockquote": "Format de texte de citation",

    "actions_tools_ai_assistant": "Assistant IA",
    "actions_tools_accessible_name_ai_assistant": "Lancer l'assistant IA",
    "actions_tools_color_picker": "Sélecteur de couleur",
    "actions_tools_accessible_name_color_picker": "Choisir une couleur de texte",

    "actions_help_label_md_syntax": "Syntaxe Markdown",
    "actions_help_accessible_name_md_syntax": "Syntaxe Markdown",
    "actions_help_label_check_for_updates": "Vérifier les mises à jour",
    "actions_help_accessible_name_check_for_updates": "Vérifier les mises à jour",
    "actions_help_label_bug_report": "Rapport de bug",
    "actions_help_accessible_name_bug_report": "Soumettre un rapport de bug",
    "actions_help_label_about": "À propos",
    "actions_help_accessible_name_about": "À propos",
}



# Dutch lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "LLM op het Apparaat",

    "module_ondevice_llm_model_exception": "Model niet gevonden fout: {error_msg}",
    "module_ondevice_llm_task_exception": "Kan modelgenerator niet initialiseren: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtTest import QTest

from notolog.ui.search_form import SearchForm
from notolog.ui.toolbar import ToolBar
from notolog.notolog_editor import NotologEditor
from notolog.settings import Settings
from notolog.enums.languages import Languages
from notolog.modules.modules import Modules

from . import test_app  # noqa: F401

import pytest


class TestToolBar:

    _test_var_click_assert_cnt = 0

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        yield settings

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Do not show actual window; return object instance only
        mocker.patch.object(NotologEditor, 'show', return_value=None)
        # Prevent resource processing, including 'process_document_images'
        mocker.patch.object(NotologEditor, 'load_content_html', return_value=None)

        # Fixture to create and return main window instance
        yield NotologEditor(screen=test_app.screens()[0])

    @pytest.fixture
    def ui_obj(self, mocker, main_window):
        # No modules
        mocker.patch.object(Modules, 'get_by_extension', return_value=[])
        # Fixture to initialize object.
        _test_action = {'type': 'action', 'weight': 1, 'name': 'toolbar_actions_label_test',
                        'theme_icon': '', 'color': 'green',
                        'label': 'Test label text', 'accessible_name': 'Test accessible name text',
                        'var_name': 'test_var', 'action': lambda: self._test_var_click_assert()}
        # TODO: Mock class to override main window actions
        ui_obj = ToolBar(parent=main_window, actions=[_test_action], refresh=False)
        yield ui_obj

    def test_ui_object_state(self, main_window, ui_obj: ToolBar, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

        assert isinstance(main_window.toolbar, ToolBar)
        assert isinstance(ui_obj, ToolBar)
        # Both objects differ in the test environment
        assert main_window.toolbar != ui_obj

        assert isinstance(ui_obj.search_form, SearchForm)

        assert ui_obj.settings.toolbar_icons == 131070

        # Variable for testing action button
        assert hasattr(ui_obj, 'test_var')

        # Verifying click action via _test_var_click_assert()
        assert self._test_var_click_assert_cnt == 0
        QTest.mouseClick(ui_obj.test_var, Qt.MouseButton.LeftButton, pos=ui_obj.test_var.rect().center())  # noqa
        assert self._test_var_click_assert_cnt == 1

    def _test_var_click_assert(self):
        assert True
        self._test_var_click_assert_cnt += 1



# Chinese lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "本地设备LLM",

    "module_ondevice_llm_config_label": "本地设备LLM模型",
    "module_ondevice_llm_config_path_label": "ONNX模型位置",
    "module_ondevice_llm_config_path_input_placeholder_text": "模型目录路径",
    "module_ondevice_llm_config_path_input_accessible_description":
        "一个带选择器的输入字段，用于指定ONNX文件所在的模型目录路径。\n"
        "支持的模型格式为ONNX，即开放神经网络交换，这是一个开放标准的机器学习模型格式。",

    "module_ondevice_llm_config_response_temperature_label": "温度：{temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "调整模型响应的随机性。较高的值产生更多变化的输出，\n"
        "而较低的值使响应更可预测。",

    "module_ondevice_llm_config_response_max_tokens_label": "最大响应令牌数",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "设置响应中接收的最大令牌数，如单词和标点，\n"
        "控制输出的长度。",

    "module_ondevice_llm_config_prompt_history_size_label": "提示历史大小",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "控制系统保留用于参考的提示历史中的条目数量。\n"
        "零值允许无限制条目。"
}



# Chinese lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "设备上的LLM",

    "module_ondevice_llm_model_exception": "模型未找到错误：{error_msg}",
    "module_ondevice_llm_task_exception": "无法初始化模型生成器：{error_msg}",
}



# Swedish lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "LLM på Enhet",

    "module_ondevice_llm_config_label": "LLM-modell på Enhet",
    "module_ondevice_llm_config_path_label": "Plats för ONNX-modellen",
    "module_ondevice_llm_config_path_input_placeholder_text": "Sökväg till modellkatalogen",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Ett inmatningsfält med en väljare för att specificera sökvägen till modellkatalogen där ONNX-filerna finns.\n"
        "Stödda modeller är i ONNX-format, vilket står för Open Neural Network Exchange, en öppen standard\n"
        "för format på maskininlärningsmodeller.",

    "module_ondevice_llm_config_response_temperature_label": "Temperatur: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Justerar slumpmässigheten i modellens svar. Högre värden producerar mer varierade utdata,\n"
        "medan lägre värden resulterar i mer förutsägbara svar.",

    "module_ondevice_llm_config_response_max_tokens_label": "Maximalt antal tokens per svar",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Anger det maximala antalet tokens som kan tas emot i ett svar, inklusive ord och skiljetecken,\n"
        "och kontrollerar utdatans längd.",

    "module_ondevice_llm_config_prompt_history_size_label": "Storlek på prompt-historik",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Kontrollerar antalet poster i prompt-historiken som systemet behåller för referens.\n"
        "Ett värde på noll tillåter obegränsat antal poster."
}



# Dutch lexemes statusbar.py
lexemes = {
    "statusbar_litter_bin_label": "Prullenbak",
    "statusbar_litter_bin_accessible_name": "Toon inhoud prullenbak",

    "statusbar_save_progress_label": b'\xf0\x9f\x92\xbe',
    "statusbar_encryption_symbol_encrypted_label": b'\xf0\x9f\x94\x92',
    "statusbar_encryption_symbol_unencrypted_label": b'\xf0\x9f\x94\x93',

    "statusbar_mode_label_mode_view": "Weergavemodus",
    "statusbar_mode_label_mode_edit": "Bewerkingsmodus",
    "statusbar_mode_label_mode_source": "Bronmodus",

    "statusbar_source_label_source_markdown": "Markdown",
    "statusbar_source_label_source_html": "HTML",

    "statusbar_encryption_label": "{encryption} {icon}",
    "statusbar_encryption_label_encryption_plain": "Ongesleuteld",
    "statusbar_encryption_label_encryption_encrypted": "Versleuteld",

    "statusbar_cursor_label": "regel: {line} → kol: {column}",
    "statusbar_cursor_label_selected": "regel: {line} → kol: {column} → sel: {selected}",
    "statusbar_cursor_label_with_global": "regel: {line} → kol: {column} → pos: {position}",
    "statusbar_cursor_label_selected_with_global": "regel: {line} → kol: {column} → pos: {position} → sel: {selected}",
}



# Finnish lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Moduuli llama.cpp",

    "module_llama_cpp_config_label": "Moduuli llama.cpp",
    "module_llama_cpp_config_path_label": "Mallin sijainti",
    "module_llama_cpp_config_path_input_placeholder_text": "Valitse tai syötä mallin polku",
    "module_llama_cpp_config_path_input_accessible_description":
        "Syöttökenttä valitsimella paikallisen mallin polun määrittämiseen. Tukee GGUF-muodossa olevia malleja,\n"
        "binääritiedostomuotoa, joka on optimoitu mallien tallentamiseen GGML:n ja GGML-pohjaisten suorittimien kanssa.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF-tiedostot",

    "module_llama_cpp_config_context_window_label": "Konteksti-ikkunan koko",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Asettaa tokenien määrän, jota malli harkitsee vastauksia tuottaessaan.\n"
        "Hallitsee kuinka paljon aiempaa kontekstia käytetään.",

    "module_llama_cpp_chat_formats_label": "Keskustelumuodot",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Valitse keskustelumuoto",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Pudotusvalikko mallikeskustelujen muodon valitsemiseksi.",

    "module_llama_cpp_config_system_prompt_label": "Järjestelmäkehotus",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Syötä järjestelmäkehotuksen teksti",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Tekstikenttä järjestelmäkehotuksien syöttämiseen, jotka ohjaavat mallin vastauksia.",

    "module_llama_cpp_config_response_temperature_label": "Vastauslämpötila: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Säätää mallin vastausten satunnaisuutta. Korkeammat arvot tuottavat monipuolisempia tulosteita,\n"
        "kun taas alhaisemmat arvot johtavat ennustettavampiin vastauksiin.",

    "module_llama_cpp_config_response_max_tokens_label": "Maksimitokenit vastauksessa",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Rajoittaa mallin vastauksissa olevien tokenien määrää aina todelliseen konteksti-ikkunan rajaan asti.\n"
        "Nolla-arvo olettaa konteksti-ikkunan kapasiteetin.",

    "module_llama_cpp_config_prompt_history_size_label": "Kehotushistorian koko",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Hallitsee järjestelmän säilyttämien kehotushistorian merkintöjen määrää viitteitä varten.\n"
        "Nolla-arvo sallii rajattoman määrän merkintöjä."
}



# Finnish lexemes ai_assistant.py
lexemes = {
    # AI-assistentin dialogi
    "dialog_title": "Pyyntö tekoälylle",
    "dialog_prompt_input_placeholder_text": "Pyyntö",
    "dialog_prompt_input_accessible_description": "Syötä pyyntö antaaksesi kontekstin tekoälyavustajalle",
    "dialog_response_output_placeholder_text": "Vastausdata",
    "dialog_response_output_accessible_description": "Vastausdata tulee näkyviin tähän kenttään",
    "dialog_response_output_notice_empty_text": "Anna tekstisyöte saadaksesi vastauksen noin...",

    "dialog_usage_module_label": "MODUULI",
    "dialog_usage_model_label": "MALLI",
    "dialog_usage_tokens_label": "TOKENIT",
    "dialog_usage_tokens_prompt": "kehoite: {tokens}",
    "dialog_usage_tokens_answer": "vastaus: {tokens}",
    "dialog_usage_tokens_total": "yhteensä: {tokens}",

    "dialog_button_send_request": "Lähetä pyyntö",
    "dialog_button_stop_request": "Pysäytä",
    "dialog_button_save_history": "Tallenna keskusteluhistoria",

    "dialog_message_copy_tooltip": "Kopioi teksti",
    "dialog_message_copied_tooltip": "Kopioitu",

    "dialog_error_loading_model": "Virhe ladattaessa mukautettua mallia",
}



# English lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Module llama.cpp",

    "module_llama_cpp_model_exception": "Model not found error: {error_msg}",
    "module_llama_cpp_task_exception": "Cannot init model generator: {error_msg}",
}



# Swedish lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "LLM på enheten",

    "module_ondevice_llm_model_exception": "Modell inte hittad-fel: {error_msg}",
    "module_ondevice_llm_task_exception": "Kan inte initiera modellgenerator: {error_msg}",
}



# Dutch lexemes ai_assistant.py
lexemes = {
    # AI assistant dialog
    "dialog_title": "Verzoek aan AI",
    "dialog_prompt_input_placeholder_text": "Verzoek",
    "dialog_prompt_input_accessible_description": "Voer een verzoek in om context te geven aan de AI-assistent",
    "dialog_response_output_placeholder_text": "Responsdata",
    "dialog_response_output_accessible_description": "Responsdata zal hier getoond worden",
    "dialog_response_output_notice_empty_text": "Voer tekst in om een respons van ongeveer... te krijgen",

    "dialog_usage_module_label": "MODULE",
    "dialog_usage_model_label": "MODEL",
    "dialog_usage_tokens_label": "TOKENS",
    "dialog_usage_tokens_prompt": "prompt: {tokens}",
    "dialog_usage_tokens_answer": "antwoord: {tokens}",
    "dialog_usage_tokens_total": "totaal: {tokens}",

    "dialog_button_send_request": "Verstuur Verzoek",
    "dialog_button_stop_request": "Stop",
    "dialog_button_save_history": "Gespreksgeschiedenis opslaan",

    "dialog_message_copy_tooltip": "Tekst kopiëren",
    "dialog_message_copied_tooltip": "Gekopieerd",

    "dialog_error_loading_model": "Fout bij laden aangepast model",
}



# Chinese lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "选择颜色",

    "color_picker_color_black": "黑色",
    "color_picker_color_silver": "银色",
    "color_picker_color_gray": "灰色",
    "color_picker_color_white": "白色",
    "color_picker_color_maroon": "栗色",
    "color_picker_color_red": "红色",
    "color_picker_color_purple": "紫色",
    "color_picker_color_fuchsia": "紫红色",
    "color_picker_color_green": "绿色",
    "color_picker_color_lime": "酸橙色",
    "color_picker_color_olive": "橄榄色",
    "color_picker_color_yellow": "黄色",
    "color_picker_color_navy": "海军蓝",
    "color_picker_color_blue": "蓝色",
    "color_picker_color_teal": "水鸭色",
    "color_picker_color_aqua": "水色",
    "color_picker_color_aliceblue": "爱丽丝蓝",
    "color_picker_color_antiquewhite": "古董白",
    "color_picker_color_aquamarine": "碧绿色",
    "color_picker_color_azure": "天蓝色",
    "color_picker_color_beige": "米色",
    "color_picker_color_bisque": "桔黄色",
    "color_picker_color_blanchedalmond": "漂白杏仁",
    "color_picker_color_blueviolet": "蓝紫色",
    "color_picker_color_brown": "棕色",
    "color_picker_color_burlywood": "硬木色",
    "color_picker_color_cadetblue": "军校蓝",
    "color_picker_color_chartreuse": "查特酒绿",
    "color_picker_color_chocolate": "巧克力色",
    "color_picker_color_coral": "珊瑚色",
    "color_picker_color_cornflowerblue": "矢车菊蓝",
    "color_picker_color_cornsilk": "玉米丝色",
    "color_picker_color_crimson": "深红色",
    "color_picker_color_cyan": "青色",
    "color_picker_color_darkblue": "深蓝色",
    "color_picker_color_darkcyan": "深青色",
    "color_picker_color_darkgoldenrod": "深金黄色",
    "color_picker_color_darkgray": "深灰色",
    "color_picker_color_darkgreen": "深绿色",
    "color_picker_color_darkkhaki": "深卡其布色",
    "color_picker_color_darkmagenta": "深洋红色",
    "color_picker_color_darkolivegreen": "深橄榄绿",
    "color_picker_color_darkorange": "深橙色",
    "color_picker_color_darkorchid": "暗兰花色",
    "color_picker_color_darkred": "深红色",
    "color_picker_color_darksalmon": "深肉色",
    "color_picker_color_darkseagreen": "深海洋绿",
    "color_picker_color_darkslateblue": "深青蓝",
    "color_picker_color_darkslategray": "深石板灰",
    "color_picker_color_darkturquoise": "深绿宝石",
    "color_picker_color_darkviolet": "深紫罗兰色",
    "color_picker_color_deeppink": "深粉色",
    "color_picker_color_deepskyblue": "深天蓝色",
    "color_picker_color_dimgray": "暗灰色",
    "color_picker_color_dodgerblue": "道奇蓝",
    "color_picker_color_firebrick": "火砖色",
    "color_picker_color_floralwhite": "花的白色",
    "color_picker_color_forestgreen": "森林绿",
    "color_picker_color_gainsboro": "淡灰色",
    "color_picker_color_ghostwhite": "幽灵白",
    "color_picker_color_gold": "金色",
    "color_picker_color_goldenrod": "金麒麟色",
    "color_picker_color_greenyellow": "绿黄色",
    "color_picker_color_honeydew": "蜜瓜色",
    "color_picker_color_hotpink": "热粉色",
    "color_picker_color_indianred": "印第安红",
    "color_picker_color_indigo": "靛青",
    "color_picker_color_ivory": "象牙色",
    "color_picker_color_khaki": "卡其色",
    "color_picker_color_lavender": "薰衣草色",
    "color_picker_color_lavenderblush": "薰衣草红",
    "color_picker_color_lawngreen": "草坪绿",
    "color_picker_color_lemonchiffon": "柠檬绸",
    "color_picker_color_lightblue": "淡蓝色",
    "color_picker_color_lightcoral": "浅珊瑚色",
    "color_picker_color_lightcyan": "淡青色",
    "color_picker_color_lightgoldenrodyellow": "淡金黄色",
    "color_picker_color_lightgray": "浅灰色",
    "color_picker_color_lightgreen": "浅绿色",
    "color_picker_color_lightpink": "浅粉色",
    "color_picker_color_lightsalmon": "浅肉色",
    "color_picker_color_lightseagreen": "浅海洋绿",
    "color_picker_color_lightskyblue": "浅天蓝色",
    "color_picker_color_lightslategray": "浅石板灰",
    "color_picker_color_lightsteelblue": "浅钢蓝",
    "color_picker_color_lightyellow": "浅黄色",
    "color_picker_color_limegreen": "酸橙绿",
    "color_picker_color_linen": "亚麻色",
    "color_picker_color_magenta": "品红",
    "color_picker_color_mediumaquamarine": "中碧绿色",
    "color_picker_color_mediumblue": "中蓝色",
    "color_picker_color_mediumorchid": "中兰花紫",
    "color_picker_color_mediumpurple": "中紫色",
    "color_picker_color_mediumseagreen": "中海洋绿",
    "color_picker_color_mediumslateblue": "中暗蓝色",
    "color_picker_color_mediumspringgreen": "中春绿色",
    "color_picker_color_mediumturquoise": "中绿宝石",
    "color_picker_color_mediumvioletred": "中紫罗兰红色",
    "color_picker_color_midnightblue": "午夜蓝",
    "color_picker_color_mintcream": "薄荷奶油",
    "color_picker_color_mistyrose": "浅玫瑰色",
    "color_picker_color_moccasin": "鹿皮鞋色",
    "color_picker_color_navajowhite": "纳瓦白",
    "color_picker_color_oldlace": "旧蕾丝",
    "color_picker_color_olivedrab": "橄榄褐",
    "color_picker_color_orange": "橙色",
    "color_picker_color_orangered": "橙红色",
    "color_picker_color_orchid": "兰花紫",
    "color_picker_color_palegoldenrod": "苍麒麟",
    "color_picker_color_palegreen": "苍绿色",
    "color_picker_color_paleturquoise": "苍绿宝石",
    "color_picker_color_palevioletred": "苍紫罗兰红",
    "color_picker_color_papayawhip": "番木瓜",
    "color_picker_color_peachpuff": "桃色",
    "color_picker_color_peru": "秘鲁色",
    "color_picker_color_pink": "粉红色",
    "color_picker_color_plum": "李子色",
    "color_picker_color_powderblue": "粉蓝色",
    "color_picker_color_rebeccapurple": "丽贝卡紫",
    "color_picker_color_rosybrown": "玫瑰棕色",
    "color_picker_color_royalblue": "宝蓝色",
    "color_picker_color_saddlebrown": "鞍棕色",
    "color_picker_color_salmon": "鲜肉色",
    "color_picker_color_sandybrown": "沙棕色",
    "color_picker_color_seagreen": "海绿色",
    "color_picker_color_seashell": "海贝壳",
    "color_picker_color_sienna": "黄土赭色",
    "color_picker_color_skyblue": "天蓝色",
    "color_picker_color_slateblue": "石蓝色",
    "color_picker_color_slategray": "灰石色",
    "color_picker_color_snow": "雪白色",
    "color_picker_color_springgreen": "春绿色",
    "color_picker_color_steelblue": "钢蓝色",
    "color_picker_color_tan": "棕褐色",
    "color_picker_color_thistle": "蓟色",
    "color_picker_color_tomato": "西红柿红",
    "color_picker_color_turquoise": "绿松石",
    "color_picker_color_violet": "紫罗兰",
    "color_picker_color_wheat": "小麦色",
    "color_picker_color_whitesmoke": "白烟",
    "color_picker_color_yellowgreen": "黄绿色",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Text edit widget.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, Signal
from PySide6.QtGui import QTextCursor, QTextBlock, QFont
from PySide6.QtWidgets import QPlainTextEdit

from typing import TYPE_CHECKING, Union

from .app_config import AppConfig

import logging

if TYPE_CHECKING:
    from PySide6.QtGui import QTextBlockUserData  # noqa: F401
    from .text_block_data import TextBlockData  # noqa: F401


class EditWidget(QPlainTextEdit):
    # Class to extend or to modify of the original QPlainTextEdit

    content_set = Signal()

    def __init__(self, parent=None):
        """
        Args:
            parent (optional): Parent object
        """
        super(EditWidget, self).__init__(parent)

        # Get app's global font size
        font = QFont()
        font.setPointSize(AppConfig().get_font_size())
        self.setFont(font)

        self.logger = logging.getLogger('edit_widget')

        # Initialize storage for positions and indexes of searched text occurrences.
        self._searched_text_positions = []

        # Disable line wrapping
        # self.setLineWrapMode(QPlainTextEdit.LineWrapMode.NoWrap)
        # Disable word wrapping
        # self.setWordWrapMode(QTextOption.WrapMode.NoWrap)

    def setDocument(self, document):
        # Override setDocument() to allow additional actions like emit the document set signal
        super().setDocument(document)
        # Emit document set signal to notify listeners
        self.content_set.emit()

    def setPlainText(self, text):
        # Override setDocument() to allow additional actions like emit the document set signal
        super().setPlainText(text)
        # Emit document set signal to notify listeners
        self.content_set.emit()

    def find_block_by_number(self, block_number: int) -> Union[QTextBlock, None]:
        """
        Notice: Very resource greedy method, avoid it, refactor or change the logic.

        Args:
            block_number (int): Block number to find

        Returns:
            Union[QTextBlock, None]: Returns either a block found by number or None
        """
        cursor = QTextCursor(self.document())
        cursor.movePosition(QTextCursor.MoveOperation.Start)

        while not (cursor.atEnd()
                   # Sometimes, cursor.atEnd() doesn't work, check it
                   or cursor.block().blockNumber() == self.document().blockCount() - 1):
            current_block = cursor.block()
            user_data = current_block.userData()  # type: Union[QTextBlockUserData, TextBlockData]

            if user_data and user_data.block_number == block_number:
                return current_block

            cursor.movePosition(QTextCursor.MoveOperation.NextBlock)

        return None

    def get_current_block(self) -> QTextBlock:
        return self.textCursor().block()

    def keyPressEvent(self, event):
        """
        This method handle key press events.

        Args:
            event (QKeyEvent): Event object
        """
        if event.key() == Qt.Key.Key_Backtab:
            # Shift + Tab action to shift the text block left either on a one tab or 4 spaces.
            self.process_back_tab()
        elif event.key() == Qt.Key.Key_Return and (event.modifiers() & Qt.KeyboardModifier.ShiftModifier):
            # Process Shift + Enter combination to allow Enter-style behaviour
            self.insertPlainText("\n")
            event.accept()
        else:
            # Default behavior for all other key events
            return QPlainTextEdit.keyPressEvent(self, event)

    def process_back_tab(self) -> None:
        """
        Shift the text block left either on a one tab or 4 spaces.
        """
        find_cursor = self.textCursor()
        """
        Current position and selection
        pos - is the position right after the char located behind the cursor; end of selection if selected
        anchor - either the beginning of the selection or the same as pos
        """
        pos = find_cursor.position()
        anchor = find_cursor.anchor()  # Where a selection starts (can be the same as above)

        # Start batch editing
        find_cursor.beginEditBlock()

        # QTextCursor.MoveMode.MoveAnchor applied by default
        find_cursor.setPosition(pos)

        # Count the number of spaces to reduce
        spaces_cnt = 0

        # Start moving the cursor position one symbol backward (QTextCursor.MoveOperation.Left)
        find_cursor.setPosition(pos - 1, QTextCursor.MoveMode.KeepAnchor)
        # Get the character on that new position
        curr_char = find_cursor.document().characterAt(find_cursor.position())
        if curr_char == " ":
            spaces_cnt += 1

        # If non-space characters continue moving cursor to any space alike position
        while (not find_cursor.document().characterAt(find_cursor.position()).isspace()
               and find_cursor.positionInBlock() > 0):
            """
            KeepAnchor makes text selected if combine it with: self.setTextCursor(find_cursor)
            """
            find_cursor.movePosition(QTextCursor.MoveOperation.Left, QTextCursor.MoveMode.MoveAnchor, 1)
            if not find_cursor.document().characterAt(find_cursor.position()).isspace():
                self.setTextCursor(find_cursor)

        # if the cursor text is a space
        if str(find_cursor.selectedText()) == " ":
            # Continue while space characters and not at beginning of the line or when a limit of spaces reached
            while (str(find_cursor.selectedText()).strip() == ""
                   and find_cursor.positionInBlock() > 0
                   and spaces_cnt < 4):
                # Move cursor to the left
                find_cursor.movePosition(QTextCursor.MoveOperation.Left, QTextCursor.MoveMode.KeepAnchor, 1)
                self.setTextCursor(find_cursor)
                cursor_char = find_cursor.document().characterAt(find_cursor.position())
                # Non-space symbol
                if cursor_char != " ":
                    """
                    " "  – Space
                    "\t" – Horizontal tab
                    "\v" – Vertical tab
                    "\n" – Newline
                    "\r" – Carriage return
                    "\f" – Feed
                    """
                    # Non-space character
                    if not cursor_char.isspace():
                        # Move anchor to a new non-space position
                        find_cursor.setPosition(find_cursor.position() + 1, QTextCursor.MoveMode.MoveAnchor)
                        # Move cursor to a new position before the non-space character
                        self.setTextCursor(find_cursor)
                        # Do not process spaces
                        spaces_cnt = 0
                    elif cursor_char == "\t":
                        # Keep anchor to add the preceding spaces to selection
                        find_cursor.setPosition(find_cursor.position() + 1, QTextCursor.MoveMode.KeepAnchor)
                        # Move cursor to a new position before the tab
                        self.setTextCursor(find_cursor)
                    break
                # Count a spaces to reduce
                spaces_cnt += 1

        if spaces_cnt > 0:
            # Reduce the spaces if any exist
            find_cursor.removeSelectedText()
        elif str(find_cursor.selectedText()) == "\t":
            # The prior character is a tab, remove it
            find_cursor.removeSelectedText()
            # Move cursor further left
            find_cursor.setPosition(anchor - 1)
            # Keep anchor from previous setPosition() and continue to move further left
            find_cursor.setPosition(pos - 1, QTextCursor.MoveMode.KeepAnchor)
        else:
            # Restore position
            find_cursor.setPosition(anchor)
            find_cursor.setPosition(pos, QTextCursor.MoveMode.KeepAnchor)

        # Stop batch editing
        find_cursor.endEditBlock()

    def searched_text_count(self, searched_text, find_flags) -> int:
        """
        Counts the number of occurrences of a given text within the document.

        Args:
            searched_text (str): The text string to search for within the document.
            find_flags (QTextDocument.FindFlags): Flags to control the search behavior,
                                                  such as case sensitivity or whole word matching.

        Returns:
            int: The number of times the searched text appears in the document.
        """
        count = 0
        # Reset stored text positions from previous searches to prepare for a new search.
        self._searched_text_positions = []
        # Create a temporary cursor for the search
        # This way, the actual cursor position that the user sees will not be affected.
        temp_cursor = self.textCursor()

        # Move the temporary cursor to the start of the document
        temp_cursor.movePosition(temp_cursor.MoveOperation.Start)

        # Find and count occurrences
        while temp_cursor := self.document().find(searched_text, temp_cursor, find_flags):
            count += 1
            pos = temp_cursor.position()
            if pos not in self._searched_text_positions:
                self._searched_text_positions.append(pos)

        # Reset the stored positions if no occurrence index is found.
        if count == 0:
            self._searched_text_positions = []

        # No need to reset the visual cursor since the original cursor was not moved
        return count

    def searched_text_index(self, position) -> int:
        """
        Shows the index of the occurrence at a given cursor position within the document.

        Args:
            position (int): The cursor position within the document.

        Returns:
            int: The index of the occurrence at a given cursor position within the document.
        """

        if position in self._searched_text_positions:
            return self._searched_text_positions.index(position) + 1
        else:
            return 0

    def contextMenuEvent(self, event):
        # Create the standard context menu
        menu = self.createStandardContextMenu()

        # Modify the menu font to hint the size
        menu.setFont(self.font())

        """
        # Optionally add new actions or modify existing ones
        customAction = menu.addAction("Custom Action")
        customAction.triggered.connect(self.custom_action_triggered)
        """

        # Display the menu
        menu.exec_(event.globalPos())



# Turkish lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Renk Seç",

    "color_picker_color_black": "Siyah",
    "color_picker_color_silver": "Gümüş",
    "color_picker_color_gray": "Gri",
    "color_picker_color_white": "Beyaz",
    "color_picker_color_maroon": "Bordo",
    "color_picker_color_red": "Kırmızı",
    "color_picker_color_purple": "Mor",
    "color_picker_color_fuchsia": "Fuşya",
    "color_picker_color_green": "Yeşil",
    "color_picker_color_lime": "Açık Yeşil",
    "color_picker_color_olive": "Zeytin Yeşili",
    "color_picker_color_yellow": "Sarı",
    "color_picker_color_navy": "Lacivert",
    "color_picker_color_blue": "Mavi",
    "color_picker_color_teal": "Teal",
    "color_picker_color_aqua": "Su Mavisi",
    "color_picker_color_aliceblue": "Alice Mavisi",
    "color_picker_color_antiquewhite": "Antik Beyaz",
    "color_picker_color_aquamarine": "Akuamarin",
    "color_picker_color_azure": "Gökyüzü Mavisi",
    "color_picker_color_beige": "Bej",
    "color_picker_color_bisque": "Bisküvi",
    "color_picker_color_blanchedalmond": "Beyazlaştırılmış Badem",
    "color_picker_color_blueviolet": "Mavi Menekşe",
    "color_picker_color_brown": "Kahverengi",
    "color_picker_color_burlywood": "Koyu Kavuniçi",
    "color_picker_color_cadetblue": "Askeri Mavi",
    "color_picker_color_chartreuse": "Parlak Sarı Yeşil",
    "color_picker_color_chocolate": "Çikolata",
    "color_picker_color_coral": "Mercan",
    "color_picker_color_cornflowerblue": "Mısır Çiçeği Mavisi",
    "color_picker_color_cornsilk": "Mısır İpeği",
    "color_picker_color_crimson": "Koyu Kırmızı",
    "color_picker_color_cyan": "Cam Göbeği",
    "color_picker_color_darkblue": "Koyu Mavi",
    "color_picker_color_darkcyan": "Koyu Camgöbeği",
    "color_picker_color_darkgoldenrod": "Koyu Altın Rengi",
    "color_picker_color_darkgray": "Koyu Gri",
    "color_picker_color_darkgreen": "Koyu Yeşil",
    "color_picker_color_darkkhaki": "Koyu Haki",
    "color_picker_color_darkmagenta": "Koyu Magenta",
    "color_picker_color_darkolivegreen": "Koyu Zeytin Yeşili",
    "color_picker_color_darkorange": "Koyu Turuncu",
    "color_picker_color_darkorchid": "Koyu Orkide",
    "color_picker_color_darkred": "Koyu Kırmızı",
    "color_picker_color_darksalmon": "Koyu Somon",
    "color_picker_color_darkseagreen": "Koyu Deniz Yeşili",
    "color_picker_color_darkslateblue": "Koyu Arduvaz Mavisi",
    "color_picker_color_darkslategray": "Koyu Arduvaz Gri",
    "color_picker_color_darkturquoise": "Koyu Turkuaz",
    "color_picker_color_darkviolet": "Koyu Menekşe",
    "color_picker_color_deeppink": "Derin Pembe",
    "color_picker_color_deepskyblue": "Derin Gökyüzü Mavisi",
    "color_picker_color_dimgray": "Sisli Gri",
    "color_picker_color_dodgerblue": "Dodger Mavisi",
    "color_picker_color_firebrick": "Ateş Tuğlası",
    "color_picker_color_floralwhite": "Çiçek Beyazı",
    "color_picker_color_forestgreen": "Orman Yeşili",
    "color_picker_color_gainsboro": "Gainsboro",
    "color_picker_color_ghostwhite": "Hayalet Beyazı",
    "color_picker_color_gold": "Altın",
    "color_picker_color_goldenrod": "Altın Rengi",
    "color_picker_color_greenyellow": "Yeşil Sarı",
    "color_picker_color_honeydew": "Bal Rengi",
    "color_picker_color_hotpink": "Canlı Pembe",
    "color_picker_color_indianred": "Hindistan Kırmızısı",
    "color_picker_color_indigo": "Çivit",
    "color_picker_color_ivory": "Fildişi",
    "color_picker_color_khaki": "Haki",
    "color_picker_color_lavender": "Lavanta",
    "color_picker_color_lavenderblush": "Lavanta Pembesi",
    "color_picker_color_lawngreen": "Çim Yeşili",
    "color_picker_color_lemonchiffon": "Limon Şifon",
    "color_picker_color_lightblue": "Açık Mavi",
    "color_picker_color_lightcoral": "Açık Mercan",
    "color_picker_color_lightcyan": "Açık Camgöbeği",
    "color_picker_color_lightgoldenrodyellow": "Açık Altın Sarısı",
    "color_picker_color_lightgray": "Açık Gri",
    "color_picker_color_lightgreen": "Açık Yeşil",
    "color_picker_color_lightpink": "Açık Pembe",
    "color_picker_color_lightsalmon": "Açık Somon",
    "color_picker_color_lightseagreen": "Açık Deniz Yeşili",
    "color_picker_color_lightskyblue": "Açık Gökyüzü Mavisi",
    "color_picker_color_lightslategray": "Açık Arduvaz Gri",
    "color_picker_color_lightsteelblue": "Açık Çelik Mavisi",
    "color_picker_color_lightyellow": "Açık Sarı",
    "color_picker_color_limegreen": "Limon Yeşili",
    "color_picker_color_linen": "Keten",
    "color_picker_color_magenta": "Macenta",
    "color_picker_color_mediumaquamarine": "Orta Akuamarin",
    "color_picker_color_mediumblue": "Orta Mavi",
    "color_picker_color_mediumorchid": "Orta Orkide",
    "color_picker_color_mediumpurple": "Orta Mor",
    "color_picker_color_mediumseagreen": "Orta Deniz Yeşili",
    "color_picker_color_mediumslateblue": "Orta Arduvaz Mavisi",
    "color_picker_color_mediumspringgreen": "Orta İlkbahar Yeşili",
    "color_picker_color_mediumturquoise": "Orta Turkuaz",
    "color_picker_color_mediumvioletred": "Orta Menekşe Kırmızısı",
    "color_picker_color_midnightblue": "Gece Mavisi",
    "color_picker_color_mintcream": "Nane Kreması",
    "color_picker_color_mistyrose": "Sisli Gül",
    "color_picker_color_moccasin": "Mokasen",
    "color_picker_color_navajowhite": "Navajo Beyazı",
    "color_picker_color_oldlace": "Eski Dantel",
    "color_picker_color_olivedrab": "Zeytin Yeşili",
    "color_picker_color_orange": "Turuncu",
    "color_picker_color_orangered": "Turuncu Kırmızı",
    "color_picker_color_orchid": "Orkide",
    "color_picker_color_palegoldenrod": "Soluk Altın Rengi",
    "color_picker_color_palegreen": "Soluk Yeşil",
    "color_picker_color_paleturquoise": "Soluk Turkuaz",
    "color_picker_color_palevioletred": "Soluk Menekşe Kırmızısı",
    "color_picker_color_papayawhip": "Papaya Kreması",
    "color_picker_color_peachpuff": "Şeftali Puf",
    "color_picker_color_peru": "Peru",
    "color_picker_color_pink": "Pembe",
    "color_picker_color_plum": "Erik",
    "color_picker_color_powderblue": "Toz Mavi",
    "color_picker_color_rebeccapurple": "Rebecca Moru",
    "color_picker_color_rosybrown": "Gül Kahverengi",
    "color_picker_color_royalblue": "Kraliyet Mavisi",
    "color_picker_color_saddlebrown": "Eyer Kahverengi",
    "color_picker_color_salmon": "Somon",
    "color_picker_color_sandybrown": "Kum Kahverengi",
    "color_picker_color_seagreen": "Deniz Yeşili",
    "color_picker_color_seashell": "Deniz Kabuğu",
    "color_picker_color_sienna": "Siyenna",
    "color_picker_color_skyblue": "Gökyüzü Mavisi",
    "color_picker_color_slateblue": "Arduvaz Mavisi",
    "color_picker_color_slategray": "Arduvaz Gri",
    "color_picker_color_snow": "Kar",
    "color_picker_color_springgreen": "Bahar Yeşili",
    "color_picker_color_steelblue": "Çelik Mavisi",
    "color_picker_color_tan": "Bronz",
    "color_picker_color_thistle": "Devedikeni",
    "color_picker_color_tomato": "Domates",
    "color_picker_color_turquoise": "Turkuaz",
    "color_picker_color_violet": "Menekşe",
    "color_picker_color_wheat": "Buğday",
    "color_picker_color_whitesmoke": "Beyaz Duman",
    "color_picker_color_yellowgreen": "Sarı Yeşil",
}



# Hindi lexemes common.py
lexemes = {
    "module_llama_cpp_name": "मॉड्यूल llama.cpp",

    "module_llama_cpp_model_exception": "मॉडल नहीं मिलने की त्रुटि: {error_msg}",
    "module_llama_cpp_task_exception": "मॉडल जनरेटर प्रारंभ नहीं किया जा सकता: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: File header class.
- Functionality: Stores and retrieves required and optional file data, as well as file state and parameters.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import datetime
import logging
import json
import re
import os

from typing import Any, Union, Iterable
from json import JSONDecodeError

from .helpers.file_helper import read_file
from .encrypt.enc_helper import EncHelper

from .exceptions.file_header_empty_exception import FileHeaderEmptyException


class FileHeader:
    """
    File header stores the data about the file and its contents.
    Header is a line of data surrounded by HTML-comment tags to avoid having it visible on external resources.

    To avoid hassle the header is only appears in files created within the Notolog editor.
    Anyway, a header is required for encrypted files as it contains such important info as salt or other file data.
    """

    HEADER_TPL = '<!-- %s -->'

    def __init__(self):
        super(FileHeader, self).__init__()

        self.logger = logging.getLogger('header')

        self.logger.debug('Header helper is activated')

        self.header = None

    def get_new(self, is_enc: bool = False):
        if not self.header:
            self.header = self.generate(is_enc=is_enc)
        else:
            self.logger.warning('Attempt to create new file header on top of existing one!')
        return self

    def generate(self, is_enc: bool = False) -> json:
        created = updated = datetime.datetime.now()
        header = {'notolog.app': {'created': created, 'updated': updated}}
        if is_enc:
            header['notolog.app'].update(self.generate_enc())
        return header

    def generate_enc(self) -> json:
        return {'enc': {'slt': EncHelper.generate_salt(), 'itr': EncHelper.get_default_iterations(), 'hint': ''}}

    def set_encrypted(self):
        self.header['notolog.app'].update(self.generate_enc())

    def is_valid(self) -> bool:
        return (self.header is not None
                and isinstance(self.header, Iterable)
                and 'notolog.app' in self.header)

    def is_file_encrypted(self) -> bool:
        if not self.is_valid():
            return False
        _header = self.header['notolog.app']
        return 'enc' in _header and isinstance(_header['enc'], dict)

    def validate_enc(self) -> None:
        """
        Ensure that the header is valid and do migrations if necessary.
        """
        pass

    def refresh(self) -> None:
        date_now = datetime.datetime.now()
        # Update only existing header
        if self.is_valid():
            self.header['notolog.app']['updated'] = date_now
            if 'created' not in self.header['notolog.app'] or self.header['notolog.app']['created'] is None:
                self.header['notolog.app']['created'] = date_now
        else:
            self.logger.debug('File header is empty')

    def get_param(self, param: Any, default: Any = None) -> Union[str, dict, None]:
        """
        Get header's param by name.
        @param param: name of the param
        @param default: default value if param is not set or not found
        @return: either string or None
        """
        return (self.header['notolog.app'][param]
                if self.is_valid() and param in self.header['notolog.app']
                else default)

    def set_param(self, param: str, value: str) -> None:
        """
        Set params to the header.
        @param param: name of the param
        @param value: value of the param
        @return: None
        """
        if self.is_valid():
            self.header['notolog.app'][param] = value
        else:
            raise FileHeaderEmptyException()

    def get_enc_param(self, param: Any) -> Union[str, None]:
        """
        Get encryption related header's param by name.
        @param param: name of the param
        @return: either string or None
        """
        enc = self.get_param('enc')
        if enc and param in enc:
            return enc[param]
        return None

    def set_enc_param(self, param: str, value: str) -> None:
        """
        Set encryption params to the header.
        @param param: name of the param
        @param value: value of the param
        @return: None
        """
        enc = self.get_param('enc')
        if enc:
            enc.update({param: value})
            self.header['notolog.app'].update({'enc': enc})
        else:
            raise FileHeaderEmptyException()

    def pack(self, content: str = None, encode: bool = False) -> str:
        header_line = repr(self)
        delimiter = '\n'

        if encode:
            delimiter = delimiter.encode("utf-8")
            if header_line and type(header_line) is not bytes:
                header_line = header_line.encode("utf-8")
            if content is not None and type(content) is not bytes:
                content = content.encode("utf-8")

        if header_line and content is not None:
            # Return header and content, even if the content is empty
            return header_line + delimiter + content
        elif content is not None:
            self.logger.debug(f'File contains content but lacks a header {self}')
            # Return content without header
            return content
        else:
            self.logger.warning(f'File contains header but lacks a content {self}')
            # Return header without content
            return header_line

    def load_file(self, file_path: str) -> tuple[Any, str]:
        file_data = read_file(file_path)
        return self.load(file_data)

    def load(self, file_data: str) -> tuple[Any, Union[str, None]]:
        try:
            file_header_line = file_data.splitlines()[0]
        except (IndexError, AttributeError):
            self.logger.debug('File header is not found')
            return self, file_data

        self.header = None
        try:
            search = re.search(self.HEADER_TPL % '(.*?)', file_header_line, re.IGNORECASE)
            file_header_json = search.group(1) if search else None
            self.header = json.loads(file_header_json)
            # Run migrations here if needed
            self.validate_enc()
        except (TypeError, JSONDecodeError):
            self.logger.debug('File header is empty')
            return self, file_data

        if self.is_valid():
            # Splitting file_data into lines
            lines = file_data.splitlines()
            if len(lines) >= 2:
                file_body = "\n".join(lines[1:])
            else:
                file_body = None
            return self, file_body

        self.logger.debug('File header is empty')  # Suppose to be a valid situation if not set yet or not created

        return self, file_data

    def read(self, file_path: str) -> Union[str, None]:
        if os.path.isfile(file_path):
            with open(file_path, encoding='utf-8') as f:
                header_line = f.readline().strip('\n')
                return header_line
        else:
            self.logger.warning('File not found "%s"' % file_path)
        return None

    def __repr__(self) -> str:
        """
        Return header data as a formatted string
        """

        if self.header:
            return self.HEADER_TPL % json.dumps(self.header, default=str)

        return ''



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: View mode Syntax Highlighter class tailored for Notolog.
- Functionality: This module includes functionality specifically needed for view mode, such as detecting strikethrough
  and 'todo' elements.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtGui import QTextBlock

from .main_highlighter import MainHighlighter
from . import TextBlockData

from typing import TYPE_CHECKING

import re

if TYPE_CHECKING:
    from typing import Match  # noqa: F401


class ViewHighlighter(MainHighlighter):
    """
    Syntax highlighter class for the view mode
    """

    theme = {
        's': {'style': 'strikethrough'},
        'todo': {'color': 'darkCyan', 'bg': {'color': 'yellow', 'pattern': Qt.BrushStyle.Dense5Pattern}},
        'inv_sep': {'font_size_ratio': 0.1},
    }
    # Keep it consistent with ini-file name, say 'viewer.ini', item prefix 'viewer_color_h1_text'
    theme_ini_prefix = 'viewer'

    """
    Elements order is matter!
    Say, i* first, b** second, bi*** third one-by-one to override prev token.
      i_, b__, bi___
      code `...`, code ```...```

    nth 0 is a whole string
    nth > 1 calculates as: start = end - length

    (?!...) - Negative Lookahead assertion
    (?<!...) - Negative Lookbehind assertion
    """
    re_rules = [
        # strikethrough
        (r'(~~(?!~|\s)[^~]*?(?<!~|\s)~~)', 1, 's', 's', False, theme['s'], None),  # between
        (r'(^|\s)(?<!~~)(~~(?!~|\s)[^~]*?)$', 2, 's_open', 's', True, theme['s'], None),
        (r'^([^~]*?[^\s~]~~)(?!~~)(?:\s|[\W^~]|$)', 1, 's_close', 's', True, theme['s'], None),
        # todos highlighting
        (r'([\s]*?)(@todo)(?=\s)', 2, 'todo', 'todo', False, theme['todo'], None),
        # invisible separator
        (r'(​)', 0, 'inv_sep', 'inv_sep', False, theme['inv_sep'], None),
    ]

    def get_regex(self, pattern: re) -> re:
        """
        Get either QRegularExpression or raw Python regex
        """
        return pattern

    def get_open_close_token_map(self):
        return [
            {'group': 's', 'open': 's_open', 'close': 's_close', 'theme': 's'}
        ]

    def highlightBlock(self, text_str):  # noqa: C901 - consider simplifying this method
        """
        Apply a syntax highlighting to each line of the text.
        https://doc.qt.io/qt-6/qsyntaxhighlighter.html#highlightBlock
        """

        # Get the current block and associated user data
        current_block = self.currentBlock()

        line_number = current_block.blockNumber()
        # Line number as it appears in the editor
        line_number_log = line_number + 1

        user_data = current_block.userData()
        if user_data is not None and type(user_data) is TextBlockData:
            """
            Skip as the highlighter may re-process the data
            """
            return

        # Create block's data storage
        user_data = TextBlockData(line_number)

        # Each line is not formatted initially
        self.clear_formatted()

        # Keep all line numbers even if no tags there
        if line_number not in self.line_tokens:
            self.line_tokens[line_number] = {}

        # Groups of tokens for correction (they have similar approach in rules)
        # oct_groups = [r.get('group') for r in self.get_open_close_token_map()]
        open_tokens = [r.get('open') for r in self.get_open_close_token_map()]
        close_tokens = [r.get('close') for r in self.get_open_close_token_map()]

        for pattern, nth, tag, group, duple, cf_data, reckon in self.rules:
            """
            Regular Python regular expression operations instead of QRegularExpression, QRegularExpressionMatchIterator
            and QRegularExpressionMatch as sometimes it doesn't work properly.
            * https://docs.python.org/3/library/re.html
            * https://doc.qt.io/qt-6/qregularexpression.html
            """
            matches = re.finditer(pattern, text_str)

            for match in matches:  # type: Match[str]
                # Get regex result position into the text string
                start = match.start(nth)
                end = match.end(nth)
                length = end - start

                # Collect line tokens only when any of them matched
                if tag not in self.line_tokens[line_number]:
                    self.line_tokens[line_number][tag] = []
                if tag != 'code' or tag not in self.get_nl_closing_tokens():
                    # The line tokens data will be reset after re-highlighting, no need to check for duplicates
                    self.line_tokens[line_number][tag].append(
                        {'start': start, 'end': end, 'length': length})  # 'in_code':...

                # Set up the tag count
                if tag in self.tokens:
                    self.tokens[tag]['cnt'] += 1
                else:
                    self.tokens[tag] = {'cnt': 1, 'o': (True if duple and tag in open_tokens else None)}

                # If the tag is duple mark it either opened or closed
                if duple:
                    if tag in open_tokens:
                        self.tokens[tag]['o'] = True
                    elif tag in close_tokens:
                        for _r in self.get_open_close_token_map():
                            if _r['group'] == group and _r['close'] == tag and _r['open'] in self.tokens:
                                self.tokens[_r['open']]['o'] = False

                """
                Check is single token inside the open close block of the same group
                """
                opened_group_token = self.get_opened_group_token(group)
                if not duple and opened_group_token is not None:
                    # Have to be there if it is not None
                    self.tokens[opened_group_token]['o'] = False
                    self.tokens[opened_group_token]['cnt'] -= 1

                """
                Non-default encodings may appear, decode them with unknown replacement.
                Also, check `surrogateescape` option.
                match.captured(nth).encode('utf-8', 'replace').decode('utf-8')
                """
                self.logger.debug(
                    'Tokens: "%s" > %s > %s (s:%d, l:%d, e:%d, n:%d)[%d] prev block state %d'
                    % (tag, self.tokens[tag], match.group(nth).encode('utf-8', 'replace'),
                       start, length, end, nth, line_number, self.previousBlockState())
                )

                # Set as formatted to skip excess processing with a similar regex
                self.set_formatted(group)

                opened = (True if tag in open_tokens else False) and self.get_opened_group_token(group) is not None
                closed = (True if tag in close_tokens else False) and self.get_opened_group_token(group) is None
                user_data.put(tag=tag, opened=opened, within=False, closed=closed, start=start, end=end)

                # Save block's data
                self.set_block_data(current_block, user_data)

        for token_data in self.get_open_close_token_map():
            """
            Process the lines located between the tags those are do not have regex matches
            """
            # Not formatted elements within the multiline block
            if (not self.is_group_formatted(token_data['group'])
                    and token_data['open'] in self.tokens
                    and self.tokens[token_data['open']]['o'] is True):
                self.set_formatted(token_data['group'])
                self.logger.debug(
                    '[%d] "%s", tag: "%s" opened: %r'
                    % (line_number_log, text_str, token_data['open'], self.tokens[token_data['open']]['o'])
                )
                tag = ('%s_within' % token_data['group'])
                user_data.put(tag=tag, opened=False, within=True, closed=False, start=0, end=len(text_str))
                # Save block's data
                self.set_block_data(current_block, user_data)

    def set_block_data(self, block: QTextBlock, data: TextBlockData) -> None:
        try:
            """
            Try to save block data to allow future processing
            """
            block.setUserData(data)
        except (TypeError, RuntimeError, ValueError) as e:
            self.logger.error('Cannot setup block data "%s" as error "%s" occurred' % (data, e))



# Greek lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Μονάδα llama.cpp",

    "module_llama_cpp_model_exception": "Σφάλμα μη εύρεσης μοντέλου: {error_msg}",
    "module_llama_cpp_task_exception": "Αδυναμία αρχικοποίησης του γεννήτριας μοντέλων: {error_msg}",
}



# Korean lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "색상 선택",

    "color_picker_color_black": "검정",
    "color_picker_color_silver": "은색",
    "color_picker_color_gray": "회색",
    "color_picker_color_white": "하얀색",
    "color_picker_color_maroon": "적갈색",
    "color_picker_color_red": "빨간색",
    "color_picker_color_purple": "보라색",
    "color_picker_color_fuchsia": "자홍색",
    "color_picker_color_green": "녹색",
    "color_picker_color_lime": "라임색",
    "color_picker_color_olive": "올리브색",
    "color_picker_color_yellow": "노란색",
    "color_picker_color_navy": "남색",
    "color_picker_color_blue": "파란색",
    "color_picker_color_teal": "청록색",
    "color_picker_color_aqua": "아쿠아",
    "color_picker_color_aliceblue": "앨리스 블루",
    "color_picker_color_antiquewhite": "앤틱 화이트",
    "color_picker_color_aquamarine": "아쿠아마린",
    "color_picker_color_azure": "청색",
    "color_picker_color_beige": "베이지",
    "color_picker_color_bisque": "비스크",
    "color_picker_color_blanchedalmond": "표백 아몬드",
    "color_picker_color_blueviolet": "파랑보라색",
    "color_picker_color_brown": "갈색",
    "color_picker_color_burlywood": "버들나무색",
    "color_picker_color_cadetblue": "카뎃블루",
    "color_picker_color_chartreuse": "샤르트뢰즈",
    "color_picker_color_chocolate": "초콜릿색",
    "color_picker_color_coral": "코랄",
    "color_picker_color_cornflowerblue": "콘플라워 블루",
    "color_picker_color_cornsilk": "콘실크",
    "color_picker_color_crimson": "진홍색",
    "color_picker_color_cyan": "청록색",
    "color_picker_color_darkblue": "어두운 파랑",
    "color_picker_color_darkcyan": "어두운 청록색",
    "color_picker_color_darkgoldenrod": "어두운 황금색",
    "color_picker_color_darkgray": "어두운 회색",
    "color_picker_color_darkgreen": "어두운 녹색",
    "color_picker_color_darkkhaki": "어두운 카키색",
    "color_picker_color_darkmagenta": "어두운 마젠타",
    "color_picker_color_darkolivegreen": "어두운 올리브 그린",
    "color_picker_color_darkorange": "어두운 주황색",
    "color_picker_color_darkorchid": "어두운 난초색",
    "color_picker_color_darkred": "어두운 빨간색",
    "color_picker_color_darksalmon": "어두운 연어색",
    "color_picker_color_darkseagreen": "어두운 바다색",
    "color_picker_color_darkslateblue": "어두운 슬레이트 블루",
    "color_picker_color_darkslategray": "어두운 슬레이트 그레이",
    "color_picker_color_darkturquoise": "어두운 터키석색",
    "color_picker_color_darkviolet": "어두운 보라색",
    "color_picker_color_deeppink": "진분홍색",
    "color_picker_color_deepskyblue": "깊은 하늘색",
    "color_picker_color_dimgray": "탁한 회색",
    "color_picker_color_dodgerblue": "도저 블루",
    "color_picker_color_firebrick": "벽돌색",
    "color_picker_color_floralwhite": "플로랄 화이트",
    "color_picker_color_forestgreen": "숲색",
    "color_picker_color_gainsboro": "게인스보로",
    "color_picker_color_ghostwhite": "고스트 화이트",
    "color_picker_color_gold": "금색",
    "color_picker_color_goldenrod": "황금빛",
    "color_picker_color_greenyellow": "녹황색",
    "color_picker_color_honeydew": "허니듀",
    "color_picker_color_hotpink": "핫핑크",
    "color_picker_color_indianred": "인디언 레드",
    "color_picker_color_indigo": "인디고",
    "color_picker_color_ivory": "상아색",
    "color_picker_color_khaki": "카키",
    "color_picker_color_lavender": "라벤더",
    "color_picker_color_lavenderblush": "라벤더 블러시",
    "color_picker_color_lawngreen": "잔디색",
    "color_picker_color_lemonchiffon": "레몬 쉬폰",
    "color_picker_color_lightblue": "밝은 파랑",
    "color_picker_color_lightcoral": "밝은 산호색",
    "color_picker_color_lightcyan": "밝은 청록색",
    "color_picker_color_lightgoldenrodyellow": "밝은 황금빛",
    "color_picker_color_lightgray": "밝은 회색",
    "color_picker_color_lightgreen": "밝은 녹색",
    "color_picker_color_lightpink": "밝은 핑크",
    "color_picker_color_lightsalmon": "밝은 연어색",
    "color_picker_color_lightseagreen": "밝은 바다색",
    "color_picker_color_lightskyblue": "밝은 하늘색",
    "color_picker_color_lightslategray": "밝은 슬레이트 그레이",
    "color_picker_color_lightsteelblue": "밝은 스틸 블루",
    "color_picker_color_lightyellow": "밝은 노란색",
    "color_picker_color_limegreen": "라임그린",
    "color_picker_color_linen": "리넨",
    "color_picker_color_magenta": "마젠타",
    "color_picker_color_mediumaquamarine": "중간 아쿠아마린",
    "color_picker_color_mediumblue": "중간 파랑",
    "color_picker_color_mediumorchid": "중간 난초",
    "color_picker_color_mediumpurple": "중간 보라",
    "color_picker_color_mediumseagreen": "중간 바다색",
    "color_picker_color_mediumslateblue": "중간 슬레이트 블루",
    "color_picker_color_mediumspringgreen": "중간 봄색",
    "color_picker_color_mediumturquoise": "중간 터키석",
    "color_picker_color_mediumvioletred": "중간 보라빛 붉은색",
    "color_picker_color_midnightblue": "자정 블루",
    "color_picker_color_mintcream": "민트 크림",
    "color_picker_color_mistyrose": "안개 장미",
    "color_picker_color_moccasin": "모카신",
    "color_picker_color_navajowhite": "나바호 화이트",
    "color_picker_color_oldlace": "오래된 레이스",
    "color_picker_color_olivedrab": "올리브 드랩",
    "color_picker_color_orange": "주황색",
    "color_picker_color_orangered": "오렌지 레드",
    "color_picker_color_orchid": "난초",
    "color_picker_color_palegoldenrod": "창백한 황금빛",
    "color_picker_color_palegreen": "창백한 녹색",
    "color_picker_color_paleturquoise": "창백한 터키석",
    "color_picker_color_palevioletred": "창백한 보라빛 붉은색",
    "color_picker_color_papayawhip": "파파야",
    "color_picker_color_peachpuff": "복숭아",
    "color_picker_color_peru": "페루",
    "color_picker_color_pink": "핑크",
    "color_picker_color_plum": "자두",
    "color_picker_color_powderblue": "파우더 블루",
    "color_picker_color_rebeccapurple": "레베카 보라색",
    "color_picker_color_rosybrown": "장미 빛 갈색",
    "color_picker_color_royalblue": "로열 블루",
    "color_picker_color_saddlebrown": "안장 갈색",
    "color_picker_color_salmon": "연어",
    "color_picker_color_sandybrown": "사막 갈색",
    "color_picker_color_seagreen": "바다색",
    "color_picker_color_seashell": "해파리",
    "color_picker_color_sienna": "시에나",
    "color_picker_color_skyblue": "하늘색",
    "color_picker_color_slateblue": "슬레이트 블루",
    "color_picker_color_slategray": "슬레이트 그레이",
    "color_picker_color_snow": "눈",
    "color_picker_color_springgreen": "봄색",
    "color_picker_color_steelblue": "강철 블루",
    "color_picker_color_tan": "황갈색",
    "color_picker_color_thistle": "엉겅퀴",
    "color_picker_color_tomato": "토마토",
    "color_picker_color_turquoise": "터키석색",
    "color_picker_color_violet": "보라색",
    "color_picker_color_wheat": "밀색",
    "color_picker_color_whitesmoke": "흰 연기",
    "color_picker_color_yellowgreen": "노랑-초록",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from notolog.ui.search_form import SearchForm
from notolog.ui.toolbar import ToolBar
from notolog.notolog_editor import NotologEditor
from notolog.settings import Settings
from notolog.enums.languages import Languages
from notolog.modules.modules import Modules

from . import test_app  # noqa: F401

import pytest


class TestSearchForm:

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        yield settings

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Do not show actual window; return object instance only
        mocker.patch.object(NotologEditor, 'show', return_value=None)
        # Prevent resource processing, including 'process_document_images'
        mocker.patch.object(NotologEditor, 'load_content_html', return_value=None)

        # Fixture to create and return main window instance
        yield NotologEditor(screen=test_app.screens()[0])

    @pytest.fixture(autouse=True)
    def toolbar_obj(self, main_window):
        # Fixture to create and return toolbar instance
        toolbar_obj = ToolBar(parent=main_window, actions=[], refresh=False)
        yield toolbar_obj

    @pytest.fixture(autouse=True)
    def ui_obj(self, mocker, main_window, toolbar_obj):
        # No modules
        mocker.patch.object(Modules, 'get_by_extension', return_value=[])
        # Fixture to initialize object.
        ui_obj = SearchForm(parent=toolbar_obj)
        yield ui_obj

    @pytest.fixture(scope="function")
    def test_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    def test_ui_object_state(self, ui_obj: SearchForm, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

        # Searched text update
        assert ui_obj.text() == ''
        ui_obj.set_text('test_text')
        assert ui_obj.text() == 'test_text'

    @pytest.mark.parametrize(
        "test_params_fixture, test_exp_params_fixture",
        [
            (('', ''), ('', '')),
            (('anything', ''), ('anything', '')),
            (('0', ''), ('0', '')),
            (('1', ''), ('1', '')),
            (('1', 'anything'), ('1', '')),
            (('anything', '3'), ('anything', '3')),
            (('2', '3'), ('2', '3')),
            (('', '4'), ('', '4')),
            (('', '0'), ('', '')),
            (('0', '5'), ('0', '5')),  # Note: The position label is not displayed
            (('0', 'five'), ('0', '')),  # Note: The position label is not displayed
        ],
        indirect=True
    )
    def test_counter(self, mocker, ui_obj, test_params_fixture, test_exp_params_fixture):
        counter_text, position_text = test_params_fixture
        exp_count_result, exp_pos_result = test_exp_params_fixture

        # Check initial occurrence values
        assert ui_obj._search_count_label.text() == ''
        assert ui_obj._search_pos_label.text() == ''

        mock_count_label_set_visible_called = mocker.patch.object(ui_obj._search_count_label, 'setVisible', return_value=None)
        mock_pos_label_set_visible_called = mocker.patch.object(ui_obj._search_pos_label, 'setVisible', return_value=None)

        # Set counter value
        ui_obj.set_counter_text(counter_text)

        # Set index position value
        ui_obj.set_position_text(position_text)

        # Check searched occurrence values
        assert ui_obj._search_count_label.text() == exp_count_result
        assert ui_obj._search_pos_label.text() == exp_pos_result

        mock_count_label_set_visible_called.assert_called_once()
        # Once during set_counter_text() and once when setting a value
        assert mock_pos_label_set_visible_called.call_count == 2



# Spanish lexemes settings_dialog.py
lexemes = {
    # Configuración
    "window_title": "Configuración",

    "button_close": "Cerrar",

    "tab_general": "General",
    "tab_editor_config": "Editor",
    "tab_viewer_config": "Visor",
    "tab_ai_config": "Configuración IA",

    "general_app_config_label": "Configuración de la app",
    "general_app_language_label": "Idioma",
    "general_app_language_combo_placeholder_text": "Elegir un idioma",
    "general_app_language_combo_accessible_description": "Idioma de la interfaz de la app",
    "general_app_theme_label": "Tema",
    "general_app_theme_combo_placeholder_text": "Elegir un tema",
    "general_app_theme_combo_accessible_description": "Tema de la interfaz de la app",
    "general_app_main_menu_label": "Menú Principal",
    "general_app_main_menu_checkbox": "Mostrar menú principal",
    "general_app_main_menu_checkbox_accessible_description": "Mostrar el menú desplegable principal de la aplicación",
    "general_app_font_size_label": "Tamaño de fuente: {size}pt",
    "general_app_font_size_slider_accessible_description": "Ajustar el tamaño de fuente global de la aplicación",

    "general_statusbar_label": "Barra de estado",
    "general_statusbar_show_global_cursor_position_checkbox": "Mostrar la posición global del cursor",
    "general_statusbar_show_global_cursor_position_checkbox_accessible_description":
        "Mostrar la posición global del cursor en la barra de estado",

    "editor_config_label": "Configuración del editor",
    "editor_config_show_line_numbers_checkbox": "Mostrar números de línea",
    "editor_config_show_line_numbers_checkbox_accessible_description": "Mostrar números de línea en el editor",

    "viewer_config_label": "Configuración del visor",
    "viewer_config_process_emojis_checkbox": "Convertir emojis de texto en gráficos",
    "viewer_config_process_emojis_checkbox_accessible_description":
        "Convertir emojis de texto en representaciones gráficas",
    "viewer_config_highlight_todos_checkbox": "Resaltar TODOs",
    "viewer_config_highlight_todos_checkbox_accessible_description": "Resaltar etiquetas TODO en el texto",
    "viewer_config_open_link_confirmation_checkbox": "Requerir confirmación para abrir enlaces",
    "viewer_config_open_link_confirmation_checkbox_accessible_description":
        "Solicitar confirmación antes de abrir enlaces",
    "viewer_config_save_resources_checkbox": "Guardar automáticamente imágenes externas en el disco",
    "viewer_config_save_resources_checkbox_accessible_description":
        "Guarda automáticamente copias de imágenes externas en el disco para acceso sin conexión.",

    "ai_config_inference_module_label": "Módulo de inferencia",
    "ai_config_inference_module_names_combo_label": "Módulo de inferencia activo",
    "ai_config_inference_module_names_combo_placeholder_text": "Elegir módulo",
    "ai_config_inference_module_names_combo_accessible_description":
        "Seleccione de los módulos de inferencia de IA disponibles para operar con el Asistente de IA.\n"
        "Las opciones incluyen modelos de lenguaje de gran tamaño (LLM) con procesamiento en tiempo real,\n"
        "o funcionalidades basadas en API.",

    "ai_config_base_label": "Parámetros básicos",
    "ai_config_multi_turn_dialogue_checkbox": "Diálogo de múltiples turnos con memoria conversacional",
    "ai_config_multi_turn_dialogue_checkbox_accessible_description":
        "Habilite un diálogo de múltiples turnos que conserve la última indicación para memoria conversacional.\n"
        "Cuando está desactivado, solo el nuevo mensaje y la indicación del sistema influyen en la respuesta.",
    "ai_config_convert_to_md_checkbox": "Convertir el resultado a Markdown",
    "ai_config_convert_to_md_checkbox_accessible_description":
        "Convierta el mensaje de salida al formato Markdown.",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QMainWindow

from notolog.app_config import AppConfig
from notolog.ui.ai_assistant.ai_assistant import AIAssistant
from notolog.ui.ai_assistant.ai_message_label import AIMessageLabel
from notolog.settings import Settings
from notolog.enums.languages import Languages
from notolog.helpers.clipboard_helper import ClipboardHelper
from notolog.helpers.tooltip_helper import TooltipHelper

from . import test_app  # noqa: F401

import pytest
import logging


class TestAIMessageLabel:

    @pytest.fixture(scope="function", autouse=True)
    def test_obj_app_config(self, mocker):
        # Mock AppConfig's get_logger_level method to suppress logging during tests.
        mocker.patch.object(AppConfig, 'get_logger_level', return_value=logging.NOTSET)

        # Create an AppConfig instance and set it to test mode.
        _app_config = AppConfig()
        _app_config.set_test_mode(True)

        yield _app_config

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        # Remember to pass the fixture to the other fixtures (check ui_obj())
        yield settings

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Fixture to create and return main window instance
        yield QMainWindow()

    @pytest.fixture(autouse=True)
    def ai_assistant_obj(self, main_window):
        # Fixture to initialize object.
        yield AIAssistant(parent=main_window)

    @pytest.fixture(autouse=True)
    def ui_obj(self, settings_obj, ai_assistant_obj, request):
        """
        Testing object fixture.
        May contain minimal logic for testing purposes.
        """

        # Retrieve parameter values from the test request.
        text = request.param if hasattr(request, 'param') else None

        yield AIMessageLabel(parent=ai_assistant_obj, text=text, settings=settings_obj)

    @pytest.fixture(scope="function")
    def test_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        """
        Fixture to pass params and get expected results.
        This method is used only for passing params via pytest fixture.
        """
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    def test_ui_object_state(self, ui_obj: AIMessageLabel, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

    @pytest.mark.parametrize(
        "test_params_fixture",
        [
            None,
            '',
            'Test Message',
        ],
        indirect=True
    )
    def test_copy_content(self, mocker, ui_obj: AIMessageLabel, test_params_fixture):
        # Extract the test method name and expected parameters
        text_to_copy = test_params_fixture

        mocker.patch.object(ui_obj, 'get_text_to_copy', return_value=text_to_copy)

        mock_clipboard_helper_set_text = mocker.patch.object(ClipboardHelper, 'set_text', return_value=None)
        mock_tooltip_helper_show_tooltip = mocker.patch.object(TooltipHelper, 'show_tooltip')

        # Run the method under test
        ui_obj.copy_content()

        mock_clipboard_helper_set_text.assert_called_once_with(text_to_copy)
        mock_tooltip_helper_show_tooltip.assert_called_once()
        assert "text" in str(mock_tooltip_helper_show_tooltip.call_args)
        assert "QPushButton" in str(mock_tooltip_helper_show_tooltip.call_args)

    @pytest.mark.parametrize(
        "ui_obj, test_params_fixture, test_exp_params_fixture",
        [
            (None, False, ''),
            (None, True, ''),
            ('', False, ''),
            ('', True, ''),
            ("Test Message", False, "Test Message"),
            ("Test Message", True, "Test Message"),
            ("<b>Test Message</b>", False, "<b>Test Message</b>"),
            ("<b>Test Message</b>", True, "Test Message"),
            ("**Test Message**", False, "**Test Message**"),
            ("**Test Message**", True, "**Test Message**"),
        ],
        indirect=True
    )
    def test_get_text_to_copy(self, settings_obj: Settings, ai_assistant_obj: AIAssistant, ui_obj: AIMessageLabel,
                              test_params_fixture, test_exp_params_fixture):
        # Check if the result needs to be converted from HTML to raw text.
        convert_to_md = test_params_fixture

        settings_obj.ai_config_convert_to_md = convert_to_md

        # Run the method under test
        result = ui_obj.get_text_to_copy()

        assert result == test_exp_params_fixture



# German lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Modul llama.cpp",

    "module_llama_cpp_model_exception": "Modell nicht gefunden Fehler: {error_msg}",
    "module_llama_cpp_task_exception": "Modellgenerator kann nicht initialisiert werden: {error_msg}",
}



# Swedish lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "API-URL",
    "module_openai_api_url_input_placeholder_text": "API-URL",
    "module_openai_api_url_input_accessible_description":
        "URL:en för OpenAI API är adressen till API-slutpunkten, som kan variera beroende på tjänst och version.\n"
        "AI-assistenten använder den för funktioner som konversationschatt eller textkompletteringar.\n"
        "Se den officiella dokumentationen för OpenAI API för att få den aktuella URL:en.",
    "module_openai_api_key_label": "API-nyckel",
    "module_openai_api_key_input_placeholder_text": "API-nyckel",
    "module_openai_api_key_input_accessible_description":
        "OpenAI API-nyckeln är en hemlig token som används för att autentisera begäranden till API-slutpunkten.",
    "module_openai_api_supported_models_label": "Stödda modeller",
    "module_openai_api_model_names_combo_placeholder_text": "Välj en modell",
    "module_openai_api_model_names_combo_accessible_description":
        "Välj från stödda modeller för chattkonversationer.",

    "module_openai_api_base_system_prompt_label": "Systemprompt",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "Bas systemprompt som föregår varje förfrågan",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "En bas systemprompt som föregår varje förfrågan.\n"
        "Vanligtvis enkel text med instruktioner eller rollkarakteristika.",

    "module_openai_api_base_response_temperature_label": "Temperatur: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "Justerar slumpmässigheten i modellens svar. Högre värden ger mer varierade utdata, "
        "medan lägre värden gör svaren mer förutsägbara.",

    "module_openai_api_base_response_max_tokens_label": "Maximalt antal respons-token",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "Maximalt antal token som tas emot i ett svar, såsom ord och skiljetecken, "
        "styr längden på utdatan.",

    "module_openai_api_config_prompt_history_size_label": "Storlek på prompt-historik",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "Kontrollerar antalet poster i prompt-historiken som systemet behåller för referens.\n"
        "Ett värde på noll tillåter obegränsade poster."
}



# Finnish lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "Valitse väri",

    "color_picker_color_black": "Musta",
    "color_picker_color_silver": "Hopea",
    "color_picker_color_gray": "Harmaa",
    "color_picker_color_white": "Valkoinen",
    "color_picker_color_maroon": "Kastanjanruskea",
    "color_picker_color_red": "Punainen",
    "color_picker_color_purple": "Purppura",
    "color_picker_color_fuchsia": "Fuksia",
    "color_picker_color_green": "Vihreä",
    "color_picker_color_lime": "Limenvihreä",
    "color_picker_color_olive": "Oliivinvihreä",
    "color_picker_color_yellow": "Keltainen",
    "color_picker_color_navy": "Laivastonsininen",
    "color_picker_color_blue": "Sininen",
    "color_picker_color_teal": "Sinivihreä",
    "color_picker_color_aqua": "Turkoosi",
    "color_picker_color_aliceblue": "Alicantensininen",
    "color_picker_color_antiquewhite": "Antiikinvalkoinen",
    "color_picker_color_aquamarine": "Akvamariini",
    "color_picker_color_azure": "Atsolansininen",
    "color_picker_color_beige": "Beesi",
    "color_picker_color_bisque": "Bisque",
    "color_picker_color_blanchedalmond": "Mantelinvaalea",
    "color_picker_color_blueviolet": "Sinivioletti",
    "color_picker_color_brown": "Ruskea",
    "color_picker_color_burlywood": "Ruskeankeltainen",
    "color_picker_color_cadetblue": "Kadetinsininen",
    "color_picker_color_chartreuse": "Chartreuse",
    "color_picker_color_chocolate": "Suklaanruskea",
    "color_picker_color_coral": "Koralli",
    "color_picker_color_cornflowerblue": "Ruiskukansininen",
    "color_picker_color_cornsilk": "Riisinkeltainen",
    "color_picker_color_crimson": "Karmiininpunainen",
    "color_picker_color_cyan": "Syanoosi",
    "color_picker_color_darkblue": "Tummansininen",
    "color_picker_color_darkcyan": "Tumma syaani",
    "color_picker_color_darkgoldenrod": "Tumma kultapiisku",
    "color_picker_color_darkgray": "Tummanharmaa",
    "color_picker_color_darkgreen": "Tummanvihreä",
    "color_picker_color_darkkhaki": "Tumma khaki",
    "color_picker_color_darkmagenta": "Tumma magenta",
    "color_picker_color_darkolivegreen": "Tumma oliivinvihreä",
    "color_picker_color_darkorange": "Tumma oranssi",
    "color_picker_color_darkorchid": "Tumma orkidea",
    "color_picker_color_darkred": "Tummanpunainen",
    "color_picker_color_darksalmon": "Tumma lohenpunainen",
    "color_picker_color_darkseagreen": "Tumma merenvihreä",
    "color_picker_color_darkslateblue": "Tumma liuskekivisininen",
    "color_picker_color_darkslategray": "Tumma liuskekiviharmaa",
    "color_picker_color_darkturquoise": "Tumma turkoosi",
    "color_picker_color_darkviolet": "Tumma violetti",
    "color_picker_color_deeppink": "Syvä pinkki",
    "color_picker_color_deepskyblue": "Syvä taivaansininen",
    "color_picker_color_dimgray": "Himmeä harmaa",
    "color_picker_color_dodgerblue": "Kiertäjänsininen",
    "color_picker_color_firebrick": "Tiilenpunainen",
    "color_picker_color_floralwhite": "Kukkavalkoinen",
    "color_picker_color_forestgreen": "Metsänvihreä",
    "color_picker_color_gainsboro": "Gainsboronharmaa",
    "color_picker_color_ghostwhite": "Haamunvalkoinen",
    "color_picker_color_gold": "Kulta",
    "color_picker_color_goldenrod": "Kultapiisku",
    "color_picker_color_greenyellow": "Vihreänkeltainen",
    "color_picker_color_honeydew": "Hunajameloni",
    "color_picker_color_hotpink": "Kuuma pinkki",
    "color_picker_color_indianred": "Intianpunainen",
    "color_picker_color_indigo": "Indigo",
    "color_picker_color_ivory": "Norsunluu",
    "color_picker_color_khaki": "Khaki",
    "color_picker_color_lavender": "Laventeli",
    "color_picker_color_lavenderblush": "Laventelinhehku",
    "color_picker_color_lawngreen": "Ruohonvihreä",
    "color_picker_color_lemonchiffon": "Sitruunankeltainen",
    "color_picker_color_lightblue": "Vaaleansininen",
    "color_picker_color_lightcoral": "Vaalea koralli",
    "color_picker_color_lightcyan": "Vaalea syaani",
    "color_picker_color_lightgoldenrodyellow": "Vaalea kultapiiskunkeltainen",
    "color_picker_color_lightgray": "Vaaleanharmaa",
    "color_picker_color_lightgreen": "Vaaleanvihreä",
    "color_picker_color_lightpink": "Vaalea pinkki",
    "color_picker_color_lightsalmon": "Vaalea lohenpunainen",
    "color_picker_color_lightseagreen": "Vaalea merenvihreä",
    "color_picker_color_lightskyblue": "Vaalea taivaansininen",
    "color_picker_color_lightslategray": "Vaalea liuskekiviharmaa",
    "color_picker_color_lightsteelblue": "Vaalea teräksensininen",
    "color_picker_color_lightyellow": "Vaaleankeltainen",
    "color_picker_color_limegreen": "Limen vihreä",
    "color_picker_color_linen": "Pellavainen",
    "color_picker_color_magenta": "Magenta",
    "color_picker_color_mediumaquamarine": "Keskiakvamariini",
    "color_picker_color_mediumblue": "Keskisininen",
    "color_picker_color_mediumorchid": "Keskiorkidea",
    "color_picker_color_mediumpurple": "Keskipurppura",
    "color_picker_color_mediumseagreen": "Keskimerenvihreä",
    "color_picker_color_mediumslateblue": "Keskiliuskekivisininen",
    "color_picker_color_mediumspringgreen": "Keskikeväänvihreä",
    "color_picker_color_mediumturquoise": "Keskiturkoosi",
    "color_picker_color_mediumvioletred": "Keskivioletinpunainen",
    "color_picker_color_midnightblue": "Keskiviikonsininen",
    "color_picker_color_mintcream": "Minttukerma",
    "color_picker_color_mistyrose": "Sumunpunainen",
    "color_picker_color_moccasin": "Mokkasiini",
    "color_picker_color_navajowhite": "Navajo-valkoinen",
    "color_picker_color_oldlace": "Vanha pitsi",
    "color_picker_color_olivedrab": "Oliivinharmaa",
    "color_picker_color_orange": "Oranssi",
    "color_picker_color_orangered": "Oranssinpunainen",
    "color_picker_color_orchid": "Orkidea",
    "color_picker_color_palegoldenrod": "Haalea kultapiisku",
    "color_picker_color_palegreen": "Haaleanvihreä",
    "color_picker_color_paleturquoise": "Haalea turkoosi",
    "color_picker_color_palevioletred": "Haalean violetinpunainen",
    "color_picker_color_papayawhip": "Papaijakerma",
    "color_picker_color_peachpuff": "Persikka",
    "color_picker_color_peru": "Perunpunainen",
    "color_picker_color_pink": "Pinkki",
    "color_picker_color_plum": "Luumu",
    "color_picker_color_powderblue": "Puuterinsininen",
    "color_picker_color_rebeccapurple": "Rebekanpurppura",
    "color_picker_color_rosybrown": "Ruusunruskea",
    "color_picker_color_royalblue": "Kuninkaansininen",
    "color_picker_color_saddlebrown": "Satulanruskea",
    "color_picker_color_salmon": "Lohenpunainen",
    "color_picker_color_sandybrown": "Hiekanruskea",
    "color_picker_color_seagreen": "Merenvihreä",
    "color_picker_color_seashell": "Simpukankuori",
    "color_picker_color_sienna": "Siena",
    "color_picker_color_skyblue": "Taivaansininen",
    "color_picker_color_slateblue": "Liuskekivisininen",
    "color_picker_color_slategray": "Liuskekiviharmaa",
    "color_picker_color_snow": "Lumi",
    "color_picker_color_springgreen": "Kevätvihreä",
    "color_picker_color_steelblue": "Teräksensininen",
    "color_picker_color_tan": "Parkkinahka",
    "color_picker_color_thistle": "Ohdake",
    "color_picker_color_tomato": "Tomaatti",
    "color_picker_color_turquoise": "Turkoosi",
    "color_picker_color_violet": "Violetti",
    "color_picker_color_wheat": "Vehnä",
    "color_picker_color_whitesmoke": "Valkoinen savu",
    "color_picker_color_yellowgreen": "Keltavihreä",
}
